@@ -50,7 +50,7 @@
 // Limit types to practical use cases.
 #define LIMIT_TYPES 1
 
-CPL_CVSID("$Id: gdalpansharpen.cpp 36763 2016-12-09 22:10:55Z rouault $");
+CPL_CVSID("$Id: gdalpansharpen.cpp 9ff327806cd64df6d73a6c91f92d12ca0c5e07df 2018-04-07 20:25:06 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                     GDALCreatePansharpenOptions()                    */
@@ -87,7 +87,7 @@ GDALPansharpenOptions * GDALCreatePansharpenOptions()
 
 void GDALDestroyPansharpenOptions( GDALPansharpenOptions* psOptions )
 {
-    if( psOptions == NULL )
+    if( psOptions == nullptr )
         return;
     CPLFree(psOptions->padfWeights);
     CPLFree(psOptions->pahInputSpectralBands);
@@ -162,9 +162,9 @@ GDALPansharpenOptions* GDALClonePansharpenOptions(
  * The object is ready to be used after Initialize() has been called.
  */
 GDALPansharpenOperation::GDALPansharpenOperation() :
-    psOptions(NULL),
+    psOptions(nullptr),
     bPositiveWeights(TRUE),
-    poThreadPool(NULL),
+    poThreadPool(nullptr),
     nKernelRadius(0)
 {}
 
@@ -196,7 +196,7 @@ GDALPansharpenOperation::~GDALPansharpenOperation()
 CPLErr
 GDALPansharpenOperation::Initialize( const GDALPansharpenOptions* psOptionsIn )
 {
-    if( psOptionsIn->hPanchroBand == NULL )
+    if( psOptionsIn->hPanchroBand == nullptr )
     {
         CPLError(CE_Failure, CPLE_AppDefined, "hPanchroBand not set");
         return CE_Failure;
@@ -207,7 +207,7 @@ GDALPansharpenOperation::Initialize( const GDALPansharpenOptions* psOptionsIn )
                  "No input spectral bands defined");
         return CE_Failure;
     }
-    if( psOptionsIn->padfWeights == NULL ||
+    if( psOptionsIn->padfWeights == nullptr ||
         psOptionsIn->nWeightCount != psOptionsIn->nInputSpectralBands )
     {
         CPLError(CE_Failure, CPLE_AppDefined,
@@ -233,7 +233,7 @@ GDALPansharpenOperation::Initialize( const GDALPansharpenOptions* psOptionsIn )
         }
         if( bSameDataset )
         {
-            if( GDALGetBandDataset(hBand) == NULL ||
+            if( GDALGetBandDataset(hBand) == nullptr ||
                 GDALGetBandDataset(hBand) != GDALGetBandDataset(hRefBand) )
             {
                 anInputBands.resize(0);
@@ -325,7 +325,7 @@ GDALPansharpenOperation::Initialize( const GDALPansharpenOptions* psOptionsIn )
         if( bNeedToWrapInVRT )
         {
             // Wrap spectral bands in a VRT if they don't have the nodata value.
-            VRTDataset* poVDS = NULL;
+            VRTDataset* poVDS = nullptr;
             for( int i = 0; i < psOptions->nInputSpectralBands; i++ )
             {
                 GDALRasterBand* poSrcBand = aMSBands[i];
@@ -336,11 +336,11 @@ GDALPansharpenOperation::Initialize( const GDALPansharpenOptions* psOptionsIn )
                 }
                 if( !anInputBands.empty() )
                     anInputBands[i] = i + 1;
-                poVDS->AddBand(poSrcBand->GetRasterDataType(), NULL);
+                poVDS->AddBand(poSrcBand->GetRasterDataType(), nullptr);
                 VRTSourcedRasterBand* poVRTBand =
                     dynamic_cast<VRTSourcedRasterBand*>(
                         poVDS->GetRasterBand(i + 1));
-                if( poVRTBand == NULL )
+                if( poVRTBand == nullptr )
                     return CE_Failure;
                 aMSBands[i] = poVRTBand;
                 poVRTBand->SetNoDataValue(psOptions->dfNoData);
@@ -372,7 +372,7 @@ GDALPansharpenOperation::Initialize( const GDALPansharpenOptions* psOptionsIn )
     else if( nThreads == 0 )
     {
         const char* pszNumThreads =
-            CPLGetConfigOption("GDAL_NUM_THREADS", NULL);
+            CPLGetConfigOption("GDAL_NUM_THREADS", nullptr);
         if( pszNumThreads )
         {
             if( EQUAL(pszNumThreads, "ALL_CPUS") )
@@ -385,11 +385,11 @@ GDALPansharpenOperation::Initialize( const GDALPansharpenOptions* psOptionsIn )
     {
         CPLDebug("PANSHARPEN", "Using %d threads", nThreads);
         poThreadPool = new (std::nothrow) CPLWorkerThreadPool();
-        if( poThreadPool == NULL ||
-            !poThreadPool->Setup( nThreads, NULL, NULL ) )
+        if( poThreadPool == nullptr ||
+            !poThreadPool->Setup( nThreads, nullptr, nullptr ) )
         {
             delete poThreadPool;
-            poThreadPool = NULL;
+            poThreadPool = nullptr;
         }
     }
 
@@ -428,7 +428,7 @@ template<class WorkDataType, class OutDataType>
     GDALCopyWord(psOptions->dfNoData, noData);
 
     if( !(std::numeric_limits<WorkDataType>::is_integer) )
-        validValue = (WorkDataType)(noData + 1e-5);
+        validValue = static_cast<WorkDataType>(noData + 1e-5);
     else if( noData == std::numeric_limits<WorkDataType>::min() )
         validValue = std::numeric_limits<WorkDataType>::min() + 1;
     else
@@ -477,6 +477,31 @@ template<class WorkDataType, class OutDataType>
     }
 }
 
+/************************************************************************/
+/*                           ComputeFactor()                            */
+/************************************************************************/
+
+template<class T> static inline double ComputeFactor(T panValue,
+                                                     double dfPseudoPanchro)
+{
+    if( dfPseudoPanchro == 0.0 )
+        return 0.0;
+
+    return panValue / dfPseudoPanchro;
+}
+
+/************************************************************************/
+/*                           ClampAndRound()                            */
+/************************************************************************/
+
+template<class T> static inline T ClampAndRound(double dfVal, T nMaxValue)
+{
+    if( dfVal > nMaxValue )
+        return nMaxValue;
+    else
+        return static_cast<T>(dfVal + 0.5);
+}
+
 /************************************************************************/
 /*                         WeightedBrovey()                             */
 /************************************************************************/
@@ -509,10 +534,7 @@ void GDALPansharpenOperation::WeightedBrovey3(
             for( int i = 0; i < psOptions->nInputSpectralBands; i++ )
                 dfPseudoPanchro += psOptions->padfWeights[i] *
                                 pUpsampledSpectralBuffer[i * nBandValues + j];
-            if( dfPseudoPanchro != 0.0 )
-                dfFactor = pPanBuffer[j] / dfPseudoPanchro;
-            else
-                dfFactor = 0.0;
+            dfFactor = ComputeFactor(pPanBuffer[j], dfPseudoPanchro);
         }
 
         for( int i = 0; i < psOptions->nOutPansharpenedBands; i++ )
@@ -534,16 +556,17 @@ void GDALPansharpenOperation::WeightedBrovey3(
 
 #include <gdalsse_priv.h>
 
-template<int NINPUT, int NOUTPUT>
+template<class T, int NINPUT, int NOUTPUT>
 int GDALPansharpenOperation::WeightedBroveyPositiveWeightsInternal(
-                                                     const GUInt16* pPanBuffer,
-                                                     const GUInt16* pUpsampledSpectralBuffer,
-                                                     GUInt16* pDataBuf,
+                                                     const T* pPanBuffer,
+                                                     const T* pUpsampledSpectralBuffer,
+                                                     T* pDataBuf,
                                                      int nValues,
                                                      int nBandValues,
-                                                     GUInt16 nMaxValue) const
+                                                     T nMaxValue) const
 {
-    CPLAssert( NINPUT == 3 || NINPUT == 4 );
+    CPL_STATIC_ASSERT( NINPUT == 3 || NINPUT == 4 );
+    CPL_STATIC_ASSERT( NOUTPUT == 3 || NOUTPUT == 4 );
     const XMMReg4Double w0 = XMMReg4Double::Load1ValHighAndLow(psOptions->padfWeights + 0);
     const XMMReg4Double w1 = XMMReg4Double::Load1ValHighAndLow(psOptions->padfWeights + 1);
     const XMMReg4Double w2 = XMMReg4Double::Load1ValHighAndLow(psOptions->padfWeights + 2);
@@ -560,22 +583,39 @@ int GDALPansharpenOperation::WeightedBroveyPositiveWeightsInternal(
     {
         XMMReg4Double pseudoPanchro = zero;
 
-        pseudoPanchro += w0 * XMMReg4Double::Load4Val(pUpsampledSpectralBuffer + j);
-        pseudoPanchro += w1 * XMMReg4Double::Load4Val(pUpsampledSpectralBuffer + nBandValues + j);
-        pseudoPanchro += w2 * XMMReg4Double::Load4Val(pUpsampledSpectralBuffer + 2 * nBandValues + j);
+        XMMReg4Double val0 = XMMReg4Double::Load4Val(pUpsampledSpectralBuffer + 0 * nBandValues + j);
+        XMMReg4Double val1 = XMMReg4Double::Load4Val(pUpsampledSpectralBuffer + 1 * nBandValues + j);
+        XMMReg4Double val2 = XMMReg4Double::Load4Val(pUpsampledSpectralBuffer + 2 * nBandValues + j);
+        XMMReg4Double val3;
+        if( NINPUT == 4 || NOUTPUT == 4 )
+        {
+            val3 = XMMReg4Double::Load4Val(pUpsampledSpectralBuffer + 3 * nBandValues + j);
+        }
+
+        pseudoPanchro += w0 * val0;
+        pseudoPanchro += w1 * val1;
+        pseudoPanchro += w2 * val2;
         if( NINPUT == 4 )
-            pseudoPanchro += w3 * XMMReg4Double::Load4Val(pUpsampledSpectralBuffer + 3 * nBandValues + j);
+            pseudoPanchro += w3 * val3;
 
         /* Little trick to avoid use of ternary operator due to one of the branch being zero */
         XMMReg4Double factor = XMMReg4Double::And(
             XMMReg4Double::NotEquals(pseudoPanchro, zero),
             XMMReg4Double::Load4Val(pPanBuffer + j) / pseudoPanchro );
 
-        for( int i = 0; i < NOUTPUT; i++ )
+        val0 = XMMReg4Double::Min(val0 * factor, maxValue);
+        val1 = XMMReg4Double::Min(val1 * factor, maxValue);
+        val2 = XMMReg4Double::Min(val2 * factor, maxValue);
+        if( NOUTPUT == 4 )
         {
-            XMMReg4Double rawValue = XMMReg4Double::Load4Val(pUpsampledSpectralBuffer + i * nBandValues + j);
-            XMMReg4Double tmp = XMMReg4Double::Min(rawValue * factor, maxValue);
-            tmp.Store4Val(pDataBuf + i * nBandValues + j);
+            val3 = XMMReg4Double::Min(val3 * factor, maxValue);
+        }
+        val0.Store4Val(pDataBuf + 0 * nBandValues + j);
+        val1.Store4Val(pDataBuf + 1 * nBandValues + j);
+        val2.Store4Val(pDataBuf + 2 * nBandValues + j);
+        if( NOUTPUT == 4 )
+        {
+            val3.Store4Val(pDataBuf + 3 * nBandValues + j);
         }
     }
     return j;
@@ -583,14 +623,14 @@ int GDALPansharpenOperation::WeightedBroveyPositiveWeightsInternal(
 
 #else
 
-template<int NINPUT, int NOUTPUT>
+template<class T, int NINPUT, int NOUTPUT>
 int GDALPansharpenOperation::WeightedBroveyPositiveWeightsInternal(
-    const GUInt16* pPanBuffer,
-    const GUInt16* pUpsampledSpectralBuffer,
-    GUInt16* pDataBuf,
+    const T* pPanBuffer,
+    const T* pUpsampledSpectralBuffer,
+    T* pDataBuf,
     int nValues,
     int nBandValues,
-    GUInt16 nMaxValue) const
+    T nMaxValue) const
 {
     // cppcheck-suppress knownConditionTrueFalse
     CPLAssert( NINPUT == 3 || NINPUT == 4 );
@@ -630,56 +670,45 @@ int GDALPansharpenOperation::WeightedBroveyPositiveWeightsInternal(
                             pUpsampledSpectralBuffer[3 * nBandValues + j + 1];
         }
 
-        if( dfPseudoPanchro != 0.0 )
-            dfFactor = pPanBuffer[j] / dfPseudoPanchro;
-        else
-            dfFactor = 0.0;
-        if( dfPseudoPanchro2 != 0.0 )
-            dfFactor2 = pPanBuffer[j+1] / dfPseudoPanchro2;
-        else
-            dfFactor2 = 0.0;
+        dfFactor = ComputeFactor(pPanBuffer[j], dfPseudoPanchro);
+        dfFactor2 = ComputeFactor(pPanBuffer[j+1], dfPseudoPanchro2);
 
         for( int i = 0; i < NOUTPUT; i++ )
         {
-            GUInt16 nRawValue =
+            T nRawValue =
                 pUpsampledSpectralBuffer[i * nBandValues + j];
             double dfTmp = nRawValue * dfFactor;
-            if( dfTmp > nMaxValue )
-                pDataBuf[i * nBandValues + j] = nMaxValue;
-            else
-                pDataBuf[i * nBandValues + j] = (GUInt16)(dfTmp + 0.5);
+            pDataBuf[i * nBandValues + j] = ClampAndRound(dfTmp, nMaxValue);
 
-            GUInt16 nRawValue2 =
+            T nRawValue2 =
                 pUpsampledSpectralBuffer[i * nBandValues + j + 1];
             double dfTmp2 = nRawValue2 * dfFactor2;
-            if( dfTmp2 > nMaxValue )
-                pDataBuf[i * nBandValues + j + 1] = nMaxValue;
-            else
-                pDataBuf[i * nBandValues + j + 1] = (GUInt16)(dfTmp2 + 0.5);
+            pDataBuf[i * nBandValues + j + 1] = ClampAndRound(dfTmp2, nMaxValue);
         }
     }
     return j;
 }
 #endif
 
+template <class T>
 void GDALPansharpenOperation::WeightedBroveyPositiveWeights(
-    const GUInt16* pPanBuffer,
-    const GUInt16* pUpsampledSpectralBuffer,
-    GUInt16* pDataBuf,
+    const T* pPanBuffer,
+    const T* pUpsampledSpectralBuffer,
+    T* pDataBuf,
     int nValues,
     int nBandValues,
-    GUInt16 nMaxValue) const
+    T nMaxValue) const
 {
     if( psOptions->bHasNoData )
     {
-        WeightedBroveyWithNoData<GUInt16, GUInt16>
+        WeightedBroveyWithNoData<T, T>
                                 (pPanBuffer, pUpsampledSpectralBuffer,
                                  pDataBuf, nValues, nBandValues, nMaxValue);
         return;
     }
 
     if( nMaxValue == 0 )
-        nMaxValue = std::numeric_limits<GUInt16>::max();
+        nMaxValue = std::numeric_limits<T>::max();
     int j;
     if( psOptions->nInputSpectralBands == 3 &&
         psOptions->nOutPansharpenedBands == 3 &&
@@ -687,7 +716,7 @@ void GDALPansharpenOperation::WeightedBroveyPositiveWeights(
         psOptions->panOutPansharpenedBands[1] == 1 &&
         psOptions->panOutPansharpenedBands[2] == 2 )
     {
-        j = WeightedBroveyPositiveWeightsInternal<3, 3>(
+        j = WeightedBroveyPositiveWeightsInternal<T, 3, 3>(
             pPanBuffer, pUpsampledSpectralBuffer, pDataBuf, nValues,
             nBandValues, nMaxValue);
     }
@@ -698,7 +727,7 @@ void GDALPansharpenOperation::WeightedBroveyPositiveWeights(
         psOptions->panOutPansharpenedBands[2] == 2 &&
         psOptions->panOutPansharpenedBands[3] == 3 )
     {
-        j = WeightedBroveyPositiveWeightsInternal<4, 4>(
+        j = WeightedBroveyPositiveWeightsInternal<T, 4, 4>(
             pPanBuffer, pUpsampledSpectralBuffer, pDataBuf, nValues,
             nBandValues, nMaxValue);
     }
@@ -708,7 +737,7 @@ void GDALPansharpenOperation::WeightedBroveyPositiveWeights(
         psOptions->panOutPansharpenedBands[1] == 1 &&
         psOptions->panOutPansharpenedBands[2] == 2 )
     {
-        j = WeightedBroveyPositiveWeightsInternal<4, 3>(
+        j = WeightedBroveyPositiveWeightsInternal<T, 4, 3>(
             pPanBuffer, pUpsampledSpectralBuffer, pDataBuf, nValues,
             nBandValues, nMaxValue);
     }
@@ -729,34 +758,21 @@ void GDALPansharpenOperation::WeightedBroveyPositiveWeights(
                     psOptions->padfWeights[i] *
                     pUpsampledSpectralBuffer[i * nBandValues + j + 1];
             }
-            if( dfPseudoPanchro != 0.0 )
-                dfFactor = pPanBuffer[j] / dfPseudoPanchro;
-            else
-                dfFactor = 0.0;
-            if( dfPseudoPanchro2 != 0.0 )
-                dfFactor2 = pPanBuffer[j+1] / dfPseudoPanchro2;
-            else
-                dfFactor2 = 0.0;
+
+            dfFactor = ComputeFactor(pPanBuffer[j], dfPseudoPanchro);
+            dfFactor2 = ComputeFactor(pPanBuffer[j+1], dfPseudoPanchro2);
 
             for( int i = 0; i < psOptions->nOutPansharpenedBands; i++ )
             {
-                const GUInt16 nRawValue =
+                const T nRawValue =
                     pUpsampledSpectralBuffer[psOptions->panOutPansharpenedBands[i] * nBandValues + j];
                 const double dfTmp = nRawValue * dfFactor;
-                if( dfTmp > nMaxValue )
-                    pDataBuf[i * nBandValues + j] = nMaxValue;
-                else
-                    pDataBuf[i * nBandValues + j] =
-                        static_cast<GUInt16>(dfTmp + 0.5);
+                pDataBuf[i * nBandValues + j] = ClampAndRound(dfTmp, nMaxValue);
 
-                const GUInt16 nRawValue2 =
+                const T nRawValue2 =
                     pUpsampledSpectralBuffer[psOptions->panOutPansharpenedBands[i] * nBandValues + j + 1];
                 const double dfTmp2 = nRawValue2 * dfFactor2;
-                if( dfTmp2 > nMaxValue )
-                    pDataBuf[i * nBandValues + j + 1] = nMaxValue;
-                else
-                    pDataBuf[i * nBandValues + j + 1] =
-                        static_cast<GUInt16>(dfTmp2 + 0.5);
+                pDataBuf[i * nBandValues + j + 1] = ClampAndRound(dfTmp2, nMaxValue);
             }
         }
     }
@@ -767,21 +783,15 @@ void GDALPansharpenOperation::WeightedBroveyPositiveWeights(
         for( int i = 0; i < psOptions->nInputSpectralBands; i++ )
             dfPseudoPanchro += psOptions->padfWeights[i] *
                             pUpsampledSpectralBuffer[i * nBandValues + j];
-        if( dfPseudoPanchro != 0.0 )
-            dfFactor = pPanBuffer[j] / dfPseudoPanchro;
-        else
-            dfFactor = 0.0;
+        dfFactor = ComputeFactor(pPanBuffer[j], dfPseudoPanchro);
 
         for( int i = 0; i < psOptions->nOutPansharpenedBands; i++ )
         {
-            GUInt16 nRawValue =
+            T nRawValue =
                 pUpsampledSpectralBuffer[psOptions->panOutPansharpenedBands[i] *
                                          nBandValues + j];
             double dfTmp = nRawValue * dfFactor;
-            if( dfTmp > nMaxValue )
-                pDataBuf[i * nBandValues + j] = nMaxValue;
-            else
-                pDataBuf[i * nBandValues + j] = (GUInt16)(dfTmp + 0.5);
+            pDataBuf[i * nBandValues + j] = ClampAndRound(dfTmp, nMaxValue);
         }
     }
 }
@@ -807,14 +817,14 @@ GDALPansharpenOperation::WeightedBrovey(
     }
 }
 
-template<>
-void GDALPansharpenOperation::WeightedBrovey<GUInt16, GUInt16>(
-    const GUInt16* pPanBuffer,
-    const GUInt16* pUpsampledSpectralBuffer,
-    GUInt16* pDataBuf,
+template<class T>
+void GDALPansharpenOperation::WeightedBroveyGByteOrUInt16(
+    const T* pPanBuffer,
+    const T* pUpsampledSpectralBuffer,
+    T* pDataBuf,
     int nValues,
     int nBandValues,
-    GUInt16 nMaxValue ) const
+    T nMaxValue ) const
 {
     if( bPositiveWeights )
     {
@@ -824,18 +834,47 @@ void GDALPansharpenOperation::WeightedBrovey<GUInt16, GUInt16>(
     }
     else if( nMaxValue == 0 )
     {
-        WeightedBrovey3<GUInt16, GUInt16, FALSE>(
+        WeightedBrovey3<T, T, FALSE>(
                 pPanBuffer, pUpsampledSpectralBuffer, pDataBuf, nValues,
                 nBandValues, 0);
     }
     else
     {
-        WeightedBrovey3<GUInt16, GUInt16, TRUE>(
+        WeightedBrovey3<T, T, TRUE>(
             pPanBuffer, pUpsampledSpectralBuffer, pDataBuf, nValues,
             nBandValues, nMaxValue);
     }
 }
 
+
+template<>
+void GDALPansharpenOperation::WeightedBrovey<GByte, GByte>(
+    const GByte* pPanBuffer,
+    const GByte* pUpsampledSpectralBuffer,
+    GByte* pDataBuf,
+    int nValues,
+    int nBandValues,
+    GByte nMaxValue ) const
+{
+    WeightedBroveyGByteOrUInt16(pPanBuffer, pUpsampledSpectralBuffer,
+                                pDataBuf, nValues,
+                                nBandValues, nMaxValue);
+}
+
+template<>
+void GDALPansharpenOperation::WeightedBrovey<GUInt16, GUInt16>(
+    const GUInt16* pPanBuffer,
+    const GUInt16* pUpsampledSpectralBuffer,
+    GUInt16* pDataBuf,
+    int nValues,
+    int nBandValues,
+    GUInt16 nMaxValue ) const
+{
+    WeightedBroveyGByteOrUInt16(pPanBuffer, pUpsampledSpectralBuffer,
+                                pDataBuf, nValues,
+                                nBandValues, nMaxValue);
+}
+
 template<class WorkDataType> CPLErr GDALPansharpenOperation::WeightedBrovey(
     const WorkDataType* pPanBuffer,
     const WorkDataType* pUpsampledSpectralBuffer,
@@ -1011,7 +1050,7 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
                                                void *pDataBuf,
                                                GDALDataType eBufDataType )
 {
-    if( psOptions == NULL )
+    if( psOptions == nullptr )
         return CE_Failure;
 
     // TODO: Avoid allocating buffers each time.
@@ -1028,7 +1067,7 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
                             psOptions->nInputSpectralBands * nDataTypeSize));
     GByte* pPanBuffer = static_cast<GByte *>(
         VSI_MALLOC3_VERBOSE(nXSize, nYSize, nDataTypeSize));
-    if( pUpsampledSpectralBuffer == NULL || pPanBuffer == NULL )
+    if( pUpsampledSpectralBuffer == nullptr || pPanBuffer == nullptr )
     {
         VSIFree(pUpsampledSpectralBuffer);
         VSIFree(pPanBuffer);
@@ -1038,7 +1077,7 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
     CPLErr eErr =
         poPanchroBand->RasterIO(GF_Read,
                 nXOff, nYOff, nXSize, nYSize, pPanBuffer, nXSize, nYSize,
-                eWorkDataType, 0, 0, NULL);
+                eWorkDataType, 0, 0, nullptr);
     if( eErr != CE_None )
     {
         VSIFree(pUpsampledSpectralBuffer);
@@ -1112,7 +1151,7 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
             VSI_MALLOC3_VERBOSE(
                 nXSizeExtract, nYSizeExtract,
                 psOptions->nInputSpectralBands * nDataTypeSize));
-        if( pSpectralBuffer == NULL )
+        if( pSpectralBuffer == nullptr )
         {
             VSIFree(pUpsampledSpectralBuffer);
             VSIFree(pPanBuffer);
@@ -1130,7 +1169,7 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
                 nXSizeExtract, nYSizeExtract,
                 eWorkDataType,
                 static_cast<int>(anInputBands.size()), &anInputBands[0],
-                0, 0, 0, NULL);
+                0, 0, 0, nullptr);
         }
         else
         {
@@ -1146,7 +1185,7 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
                     static_cast<size_t>(i) *
                     nXSizeExtract * nYSizeExtract * nDataTypeSize,
                     nXSizeExtract, nYSizeExtract,
-                    eWorkDataType, 0, 0, NULL);
+                    eWorkDataType, 0, 0, nullptr);
             }
         }
         if( eErr != CE_None )
@@ -1159,7 +1198,7 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
 
         // Create a MEM dataset that wraps the input buffer.
         GDALDataset* poMEMDS = MEMDataset::Create("", nXSizeExtract, nYSizeExtract, 0,
-                                                  eWorkDataType, NULL);
+                                                  eWorkDataType, nullptr);
 
         char szBuffer0[64] = {};
         char szBuffer1[64] = {};
@@ -1172,11 +1211,11 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
         apszOptions[0] = szBuffer0;
         apszOptions[1] = szBuffer1;
         apszOptions[2] = szBuffer2;
-        apszOptions[3] = NULL;
+        apszOptions[3] = nullptr;
 
         for( int i = 0; i < psOptions->nInputSpectralBands; i++ )
         {
-            char szBuffer[64] = {};
+            char szBuffer[32] = {};
             int nRet = CPLPrintPointer(
                 szBuffer,
                 pSpectralBuffer +
@@ -1212,7 +1251,7 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
                               nSpectralXSize, nSpectralYSize,
                               pUpsampledSpectralBuffer, nXSize, nYSize,
                               eWorkDataType,
-                              psOptions->nInputSpectralBands, NULL,
+                              psOptions->nInputSpectralBands, nullptr,
                               0, 0, 0,
                               &sExtraArg));
         }
@@ -1297,7 +1336,7 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
                     ahJobData[i] = &(pasJobs[i]);
                 }
 #ifdef DEBUG_TIMING
-                gettimeofday(&tv, NULL);
+                gettimeofday(&tv, nullptr);
 #endif
                 poThreadPool->SubmitJobs(PansharpenResampleJobThreadFunc,
                                          ahJobData);
@@ -1367,23 +1406,23 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
             {
                 if( eWorkDataType == GDT_Byte )
                 {
-                    ClampValues(((GByte*)pUpsampledSpectralBuffer) + i * nXSize * nYSize,
+                    ClampValues(reinterpret_cast<GByte*>(pUpsampledSpectralBuffer) + i * nXSize * nYSize,
                                nXSize*nYSize,
-                               (GByte)((1 << nBitDepth)-1));
+                               static_cast<GByte>((1 << nBitDepth)-1));
                 }
                 else if( eWorkDataType == GDT_UInt16 )
                 {
-                    ClampValues(((GUInt16*)pUpsampledSpectralBuffer) + i * nXSize * nYSize,
+                    ClampValues(reinterpret_cast<GUInt16*>(pUpsampledSpectralBuffer) + i * nXSize * nYSize,
                                nXSize*nYSize,
-                               (GUInt16)((1 << nBitDepth)-1));
+                               static_cast<GUInt16>((1 << nBitDepth)-1));
                 }
 #ifndef LIMIT_TYPES
                 else if( eWorkDataType == GDT_UInt32 )
                 {
-                    ClampValues(((GUInt32*)pUpsampledSpectralBuffer) +
+                    ClampValues(reinterpret_cast<GUInt32*>(pUpsampledSpectralBuffer) +
                                 i * nXSize * nYSize,
                                 nXSize*nYSize,
-                                (GUInt32)((1 << nBitDepth)-1));
+                                (static_cast<GUInt32>((1 << nBitDepth)-1));
                 }
 #endif
             }
@@ -1392,7 +1431,7 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
 
     GUInt32 nMaxValue = (1 << nBitDepth) - 1;
 
-    double* padfTempBuffer = NULL;
+    double* padfTempBuffer = nullptr;
     GDALDataType eBufDataTypeOri = eBufDataType;
     void* pDataBufOri = pDataBuf;
     // CFloat64 is the query type used by gdallocationinfo...
@@ -1406,7 +1445,7 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
             VSI_MALLOC3_VERBOSE(
                 nXSize, nYSize,
                 psOptions->nOutPansharpenedBands * sizeof(double)));
-        if( padfTempBuffer == NULL )
+        if( padfTempBuffer == nullptr )
         {
             VSIFree(pUpsampledSpectralBuffer);
             VSIFree(pPanBuffer);
@@ -1455,7 +1494,7 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
                 ahJobData[i] = &(pasJobs[i]);
             }
 #ifdef DEBUG_TIMING
-            gettimeofday(&tv, NULL);
+            gettimeofday(&tv, nullptr);
 #endif
             poThreadPool->SubmitJobs(PansharpenJobThreadFunc, ahJobData);
             poThreadPool->WaitCompletion();
@@ -1502,11 +1541,11 @@ CPLErr GDALPansharpenOperation::ProcessRegion( int nXOff, int nYOff,
 
 void GDALPansharpenOperation::PansharpenResampleJobThreadFunc(void* pUserData)
 {
-    GDALPansharpenResampleJob* psJob = (GDALPansharpenResampleJob*) pUserData;
+    GDALPansharpenResampleJob* psJob = static_cast<GDALPansharpenResampleJob*>(pUserData);
 
 #ifdef DEBUG_TIMING
     struct timeval tv;
-    gettimeofday(&tv, NULL);
+    gettimeofday(&tv, nullptr);
     const GIntBig launch_time =
         static_cast<GIntBig>(psJob->ptv->tv_sec) * 1000000 +
         static_cast<GIntBig>(psJob->ptv->tv_usec);
@@ -1538,14 +1577,14 @@ void GDALPansharpenOperation::PansharpenResampleJobThreadFunc(void* pUserData)
                              psJob->nBufYSize,
                              psJob->eDT,
                              psJob->nBandCount,
-                             NULL,
+                             nullptr,
                              0, 0, psJob->nBandSpace,
                              &sExtraArg));
 #endif
 
 #ifdef DEBUG_TIMING
     struct timeval tv_end;
-    gettimeofday(&tv_end, NULL);
+    gettimeofday(&tv_end, nullptr);
     const GIntBig end =
         static_cast<GIntBig>(tv_end.tv_sec) * 1000000 +
         static_cast<GIntBig>(tv_end.tv_usec);
@@ -1562,11 +1601,11 @@ void GDALPansharpenOperation::PansharpenResampleJobThreadFunc(void* pUserData)
 
 void GDALPansharpenOperation::PansharpenJobThreadFunc(void* pUserData)
 {
-    GDALPansharpenJob* psJob = (GDALPansharpenJob*) pUserData;
+    GDALPansharpenJob* psJob = static_cast<GDALPansharpenJob*>(pUserData);
 
 #ifdef DEBUG_TIMING
     struct timeval tv;
-    gettimeofday(&tv, NULL);
+    gettimeofday(&tv, nullptr);
     const GIntBig launch_time =
         static_cast<GIntBig>(psJob->ptv->tv_sec) * 1000000 +
         static_cast<GIntBig>(psJob->ptv->tv_usec);
@@ -1593,7 +1632,7 @@ void GDALPansharpenOperation::PansharpenJobThreadFunc(void* pUserData)
 
 #ifdef DEBUG_TIMING
     struct timeval tv_end;
-    gettimeofday(&tv_end, NULL);
+    gettimeofday(&tv_end, nullptr);
     const GIntBig end =
         static_cast<GIntBig>(tv_end.tv_sec) * 1000000 +
         static_cast<GIntBig>(tv_end.tv_usec);
@@ -1623,51 +1662,51 @@ GDALPansharpenOperation::PansharpenChunk( GDALDataType eWorkDataType,
     switch( eWorkDataType )
     {
         case GDT_Byte:
-            eErr = WeightedBrovey((GByte*)pPanBuffer,
-                                  (GByte*)pUpsampledSpectralBuffer,
+            eErr = WeightedBrovey(reinterpret_cast<const GByte*>(pPanBuffer),
+                                  reinterpret_cast<const GByte*>(pUpsampledSpectralBuffer),
                                   pDataBuf, eBufDataType,
-                                  nValues, nBandValues, (GByte)nMaxValue);
+                                  nValues, nBandValues, static_cast<GByte>(nMaxValue));
             break;
 
         case GDT_UInt16:
-            eErr = WeightedBrovey((GUInt16*)pPanBuffer,
-                                  (GUInt16*)pUpsampledSpectralBuffer,
+            eErr = WeightedBrovey(reinterpret_cast<const GUInt16*>(pPanBuffer),
+                                  reinterpret_cast<const GUInt16*>(pUpsampledSpectralBuffer),
                                   pDataBuf, eBufDataType,
-                                  nValues, nBandValues, (GUInt16)nMaxValue);
+                                  nValues, nBandValues, static_cast<GUInt16>(nMaxValue));
             break;
 
 #ifndef LIMIT_TYPES
         case GDT_Int16:
-            eErr = WeightedBrovey((GInt16*)pPanBuffer,
-                                  (GInt16*)pUpsampledSpectralBuffer,
+            eErr = WeightedBrovey(reinterpret_cast<const GInt16*>(pPanBuffer),
+                                  reinterpret_cast<const GInt16*>(pUpsampledSpectralBuffer),
                                   pDataBuf, eBufDataType,
                                   nValues, nBandValues);
             break;
 
         case GDT_UInt32:
-            eErr = WeightedBrovey((GUInt32*)pPanBuffer,
-                                  (GUInt32*)pUpsampledSpectralBuffer,
+            eErr = WeightedBrovey(reinterpret_cast<const GUInt32*>(pPanBuffer),
+                                  reinterpret_cast<const GUInt32*>(pUpsampledSpectralBuffer),
                                   pDataBuf, eBufDataType,
                                   nValues, nBandValues, nMaxValue);
             break;
 
         case GDT_Int32:
-            eErr = WeightedBrovey((GInt32*)pPanBuffer,
-                                  (GInt32*)pUpsampledSpectralBuffer,
+            eErr = WeightedBrovey(reinterpret_cast<const GInt32*>(pPanBuffer),
+                                  reinterpret_cast<const GInt32*>(pUpsampledSpectralBuffer),
                                   pDataBuf, eBufDataType,
                                   nValues, nBandValues);
             break;
 
         case GDT_Float32:
-            eErr = WeightedBrovey((float*)pPanBuffer,
-                                  (float*)pUpsampledSpectralBuffer,
+            eErr = WeightedBrovey(reinterpret_cast<const float*>(pPanBuffer),
+                                  reinterpret_cast<const float*>(pUpsampledSpectralBuffer),
                                   pDataBuf, eBufDataType,
                                   nValues, nBandValues);
             break;
 #endif
         case GDT_Float64:
-            eErr = WeightedBrovey((double*)pPanBuffer,
-                                  (double*)pUpsampledSpectralBuffer,
+            eErr = WeightedBrovey(reinterpret_cast<const double*>(pPanBuffer),
+                                  reinterpret_cast<const double*>(pUpsampledSpectralBuffer),
                                   pDataBuf, eBufDataType,
                                   nValues, nBandValues);
             break;
@@ -1714,9 +1753,9 @@ GDALPansharpenOperationH GDALCreatePansharpenOperation(
 {
     GDALPansharpenOperation* psOperation = new GDALPansharpenOperation();
     if( psOperation->Initialize(psOptions) == CE_None )
-        return (GDALPansharpenOperationH)psOperation;
+        return reinterpret_cast<GDALPansharpenOperationH>(psOperation);
     delete psOperation;
-    return NULL;
+    return nullptr;
 }
 
 /************************************************************************/
