@@ -14,16 +14,16 @@
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  ****************************************************************************/
 
@@ -33,7 +33,7 @@
 
 #include "georaster_priv.h"
 
-CPL_CVSID("$Id: cpl_vsil_ocilob.cpp $");
+CPL_CVSID("$Id: cpl_vsil_ocilob.cpp $")
 
 // *****************************************************************************
 //                                                             WSIOCILobFSHandle
@@ -41,19 +41,17 @@ CPL_CVSID("$Id: cpl_vsil_ocilob.cpp $");
 
 class WSIOCILobFSHandle : public VSIFilesystemHandler
 {
+  public:
+    WSIOCILobFSHandle();
+    ~WSIOCILobFSHandle() override;
 
-public:
-                              WSIOCILobFSHandle();
-    virtual                  ~WSIOCILobFSHandle();
-
-    virtual VSIVirtualHandle *Open( const char *pszFilename, 
-                                    const char *pszAccess,
-                                    bool bSetError ) override;
-    virtual int               Stat( const char *pszFilename,
-                                    VSIStatBufL *pStatBuf, int nFlags ) override;
-
-private:
+    VSIVirtualHandle *Open( const char *pszFilename,
+                            const char *pszAccess,
+                            bool bSetError ) override;
+    int               Stat( const char *pszFilename,
+                            VSIStatBufL *pStatBuf, int nFlags ) override;
 
+  private:
     OWConnection*     poConnection;
     OWStatement*      poStatement;
     OCILobLocator*    phLocator;
@@ -77,18 +75,18 @@ class VSIOCILobHandle : public VSIVirtualHandle
     boolean           bUpdate;
 
   public:
-                      VSIOCILobHandle( OWConnection* poConnectionIn,
-                                       OWStatement* poStatementIn,
-                                       OCILobLocator* phLocatorIn,
-                                       boolean bUpdateIn );
-    virtual          ~VSIOCILobHandle();
+    VSIOCILobHandle( OWConnection* poConnectionIn,
+                     OWStatement* poStatementIn,
+                     OCILobLocator* phLocatorIn,
+                     boolean bUpdateIn );
+    ~VSIOCILobHandle() override;
 
-    virtual int       Seek( vsi_l_offset nOffset, int nWhence ) override;
-    virtual vsi_l_offset Tell() override;
-    virtual size_t    Read( void *pBuffer, size_t nSize, size_t nMemb ) override;
-    virtual size_t    Write( const void *pBuffer, size_t nSize, size_t nMemb ) override;
-    virtual int       Eof() override;
-    virtual int       Close() override;
+    int Seek( vsi_l_offset nOffset, int nWhence ) override;
+    vsi_l_offset Tell() override;
+    size_t Read( void *pBuffer, size_t nSize, size_t nMemb ) override;
+    size_t Write( const void *pBuffer, size_t nSize, size_t nMemb ) override;
+    int Eof() override;
+    int Close() override;
 };
 
 // ****************************************************************************
@@ -101,9 +99,9 @@ class VSIOCILobHandle : public VSIVirtualHandle
 
 WSIOCILobFSHandle::WSIOCILobFSHandle()
 {
-    poStatement  = NULL;
-    phLocator    = NULL;
-    poConnection = NULL;
+    poStatement  = nullptr;
+    phLocator    = nullptr;
+    poConnection = nullptr;
 }
 
 // -----------------------------------------------------------------------------
@@ -136,10 +134,10 @@ char** WSIOCILobFSHandle::ParseIdentificator( const char* pszFilename )
 {
     if( strncmp(pszFilename, "/vsiocilob/", strlen("/vsiocilob/") ) != 0 )
     {
-        return NULL;
+        return nullptr;
     }
 
-    char** papszParam = CSLTokenizeString2( 
+    char** papszParam = CSLTokenizeString2(
                             &pszFilename[strlen("/vsiocilob/")], ",",
                             CSLT_HONOURSTRINGS | CSLT_ALLOWEMPTYTOKENS |
                             CSLT_STRIPLEADSPACES | CSLT_STRIPENDSPACES );
@@ -147,7 +145,7 @@ char** WSIOCILobFSHandle::ParseIdentificator( const char* pszFilename )
     if( CSLCount( papszParam ) < 6 )
     {
         CSLDestroy( papszParam );
-        return NULL;
+        return nullptr;
     }
 
     return papszParam;
@@ -157,7 +155,7 @@ char** WSIOCILobFSHandle::ParseIdentificator( const char* pszFilename )
 //                                                                        Open()
 // -----------------------------------------------------------------------------
 
-VSIVirtualHandle* WSIOCILobFSHandle::Open( const char* pszFilename, 
+VSIVirtualHandle* WSIOCILobFSHandle::Open( const char* pszFilename,
                                            const char* pszAccess,
                                            bool /* bSetError*/ )
 {
@@ -165,13 +163,13 @@ VSIVirtualHandle* WSIOCILobFSHandle::Open( const char* pszFilename,
 
     if( ! papszParam )
     {
-        return NULL;
+        return nullptr;
     }
 
     if( ! EQUAL( papszParam[5], "noext" ) )
     {
         CSLDestroy( papszParam );
-        return NULL;
+        return nullptr;
     }
 
     poConnection = new OWConnection( papszParam[0],
@@ -181,21 +179,21 @@ VSIVirtualHandle* WSIOCILobFSHandle::Open( const char* pszFilename,
     if( ! poConnection->Succeeded() )
     {
         CSLDestroy( papszParam );
-        return NULL;
+        return nullptr;
     }
 
     const char *pszUpdate = "";
     boolean bUpdate = false;
 
-    if( strchr(pszAccess, 'w') != NULL ||
-        strchr(pszAccess, '+') != NULL )
+    if( strchr(pszAccess, 'w') != nullptr ||
+        strchr(pszAccess, '+') != nullptr )
     {
         pszUpdate = "for update";
         bUpdate = true;
     }
 
-    poStatement = poConnection->CreateStatement( CPLSPrintf( 
-                    "select rasterblock from %s where rasterid = %s and rownum = 1 %s", 
+    poStatement = poConnection->CreateStatement( CPLSPrintf(
+                    "select rasterblock from %s where rasterid = %s and rownum = 1 %s",
                     papszParam[3], papszParam[4], pszUpdate ) );
 
     poStatement->Define( &phLocator );
@@ -204,7 +202,7 @@ VSIVirtualHandle* WSIOCILobFSHandle::Open( const char* pszFilename,
 
     if( ! poStatement->Execute() )
     {
-        return NULL;
+        return nullptr;
     }
 
     return new VSIOCILobHandle( poConnection, poStatement, phLocator, bUpdate );
