@@ -34,7 +34,7 @@
 */
 
 /******************************************************************************
- * $Id: marfa.h 37669 2017-03-09 23:35:51Z lplesea $
+ * $Id: marfa.h 22f8ae3bf7bc3cccd970992655c63fc5254d3206 2018-04-08 20:13:05 +0200 Even Rouault $
  *
  * Project:  Meta Raster Format
  * Purpose:  MRF structures
@@ -82,6 +82,8 @@ NAMESPACE_MRF_START
 //
 #define ZFLAG_SMASK 0x1c0
 
+#define PADDING_BYTES 3
+
 // Force LERC to be included, normally off, detected in the makefile
 // #define LERC
 
@@ -125,13 +127,13 @@ struct ILSize {
         x = x_; y = y_; z = z_; c = c_; l = l_;
     }
 
-    bool operator==(const ILSize& other)
+    bool operator==(const ILSize& other) const
     {
         return ((x == other.x) && (y == other.y) && (z == other.z) &&
             (c == other.c) && (l == other.l));
     }
 
-    bool operator!=(const ILSize& other) { return !(*this == other); }
+    bool operator!=(const ILSize& other) const { return !(*this == other); }
 };
 
 std::ostream& operator<<(std::ostream &out, const ILSize& sz);
@@ -221,6 +223,15 @@ static inline unsigned long long net64(const unsigned long long x)
 #define net64(x) swab64(x)
 #endif
 
+// Count the values in a buffer that match a specific value
+template<typename T> static int MatchCount(T *buff, int sz, T val) {
+    int ncount = 0;
+    for (int i = 0; i < sz; i++)
+        if (buff[i] == val)
+            ncount++;
+    return ncount;
+}
+
 const char *CompName(ILCompression comp);
 const char *OrderName(ILOrder val);
 ILCompression CompToken(const char *, ILCompression def = IL_ERR_COMP);
@@ -236,7 +247,7 @@ CPLString PrintDouble(double d, const char *frmt = "%12.8f");
 void XMLSetAttributeVal(CPLXMLNode *parent, const char* pszName,
     const double val, const char *frmt = "%12.8f");
 CPLXMLNode *XMLSetAttributeVal(CPLXMLNode *parent,
-    const char*pszName, const ILSize &sz, const char *frmt = NULL);
+    const char*pszName, const ILSize &sz, const char *frmt = nullptr);
 void XMLSetAttributeVal(CPLXMLNode *parent,
     const char*pszName, std::vector<double> const &values);
 //
@@ -285,7 +296,7 @@ enum { SAMPLING_ERR, SAMPLING_Avg, SAMPLING_Near };
 
 GDALMRFRasterBand *newMRFRasterBand(GDALMRFDataset *, const ILImage &, int, int level = 0);
 
-class GDALMRFDataset : public GDALPamDataset {
+class GDALMRFDataset final: public GDALPamDataset {
     friend class GDALMRFRasterBand;
     friend GDALMRFRasterBand *newMRFRasterBand(GDALMRFDataset *, const ILImage &, int, int level);
 
@@ -327,14 +338,14 @@ public:
 
     virtual char **GetFileList() override;
 
-    void SetColorTable(GDALColorTable *pct) { poColorTable = pct; };
-    const GDALColorTable *GetColorTable() { return poColorTable; };
+    void SetColorTable(GDALColorTable *pct) { poColorTable = pct; }
+    const GDALColorTable *GetColorTable() { return poColorTable; }
     void SetNoDataValue(const char*);
     void SetMinValue(const char*);
     void SetMaxValue(const char*);
     CPLErr SetVersion(int version);
 
-    const CPLString GetFname() { return fname; };
+    const CPLString GetFname() { return fname; }
     // Patches a region of all the next overview, argument counts are in blocks
     virtual CPLErr PatchOverview(int BlockX, int BlockY, int Width, int Height,
         int srcLevel = 0, int recursive = false, int sampling_mode = SAMPLING_Avg);
@@ -358,10 +369,14 @@ protected:
     // Apply create options to the current dataset
     void ProcessCreateOptions(char **papszOptions);
 
+    // Called once before the parsing of the XML, should just capture the options in dataset variables
+    void ProcessOpenOptions(char **papszOptions);
+
     // Writes the XML tree as MRF.  It does not check the content
     int WriteConfig(CPLXMLNode *);
 
     // Initializes the dataset from an MRF metadata XML
+    // Options should be papszOpenOptions, but the dataset already has a member with that name
     CPLErr Initialize(CPLXMLNode *);
 
     // Do nothing, this is not possible in an MRF
@@ -398,6 +413,9 @@ protected:
     // Write a tile, the infooffset is the relative position in the index file
     virtual CPLErr WriteTile(void *buff, GUIntBig infooffset, GUIntBig size = 0);
 
+    // Custom CopyWholeRaster for Zen JPEG
+    CPLErr ZenCopy(GDALDataset *poSrc, GDALProgressFunc pfnProgress, void * pProgressData);
+
     // For versioned MRFs, add a version
     CPLErr AddVersion();
 
@@ -409,11 +427,11 @@ protected:
     GDALRWFlag IdxMode() {
         if (!ifp.FP) IdxFP();
         return ifp.acc;
-    };
+    }
     GDALRWFlag DataMode() {
         if (!dfp.FP) DataFP();
         return dfp.acc;
-    };
+    }
     GDALDataset *GetSrcDS();
 
     /*
@@ -439,12 +457,15 @@ protected:
     GIntBig idxSize; // The size of each version index, or the size of the cloned index
 
     int clonedSource; // Is it a cloned source
+    int nocopy;       // Set when initializing a caching MRF
     int bypass_cache; // Do we alter disk cache
     int mp_safe;      // Not thread safe, only multiple writers
     int hasVersions;  // Does it support versions
     int verCount;     // The last version
     int bCrystalized; // Unset only during the create process
     int spacing;      // How many spare bytes before each tile data
+    int no_errors;    // Ignore read errors
+    int missing;      // set if no_errors is set and data is missing
 
     // Freeform sticky dataset options, as a list of key-value pairs
     CPLStringList optlist;
@@ -508,15 +529,18 @@ public:
     virtual double  GetMaximum(int *) override;
 
     // MRF specific, fetch is from a remote source
-    CPLErr FetchBlock(int xblk, int yblk, void *buffer = NULL);
+    CPLErr FetchBlock(int xblk, int yblk, void *buffer = nullptr);
     // Fetch a block from a cloned MRF
-    CPLErr FetchClonedBlock(int xblk, int yblk, void *buffer = NULL);
+    CPLErr FetchClonedBlock(int xblk, int yblk, void *buffer = nullptr);
 
     // Block not stored on disk
     CPLErr FillBlock(void *buffer);
 
+    // Same, for interleaved bands, current band goes in buffer
+    CPLErr FillBlock(int xblk, int yblk, void *buffer);
+
     // de-interlace a buffer in pixel blocks
-    CPLErr RB(int xblk, int yblk, buf_mgr src, void *buffer);
+    CPLErr ReadInterleavedBlock(int xblk, int yblk, void *buffer);
 
     const char *GetOptionValue(const char *opt, const char *def) const;
     void SetAccess(GDALAccess eA) { eAccess = eA; }
@@ -533,7 +557,6 @@ protected:
     // The info about the current image, to enable R-sets
     ILImage img;
     std::vector<GDALMRFRasterBand *> overviews;
-    int overview;
 
     VSILFILE *IdxFP() { return poDS->IdxFP(); }
     GDALRWFlag IdxMode() { return poDS->IdxMode(); }
@@ -578,7 +601,7 @@ protected:
 class PNG_Codec {
 public:
     explicit PNG_Codec(const ILImage &image) : img(image),
-        PNGColors(NULL), PNGAlpha(NULL), PalSize(0), TransSize(0), deflate_flags(0) {};
+        PNGColors(nullptr), PNGAlpha(nullptr), PalSize(0), TransSize(0), deflate_flags(0) {}
 
     virtual ~PNG_Codec() {
         CPLFree(PNGColors);
@@ -616,7 +639,7 @@ protected:
 
 class JPEG_Codec {
 public:
-    explicit JPEG_Codec(const ILImage &image) : img(image), sameres(FALSE), rgb(FALSE), optimize(false) {};
+    explicit JPEG_Codec(const ILImage &image) : img(image), sameres(FALSE), rgb(FALSE), optimize(false) {}
 
     CPLErr CompressJPEG(buf_mgr &dst, buf_mgr &src);
     CPLErr DecompressJPEG(buf_mgr &dst, buf_mgr &src);
@@ -642,7 +665,7 @@ class JPEG_Band : public GDALMRFRasterBand {
     friend class GDALMRFDataset;
 public:
     JPEG_Band(GDALMRFDataset *pDS, const ILImage &image, int b, int level);
-    virtual ~JPEG_Band() {};
+    virtual ~JPEG_Band() {}
 
 protected:
     virtual CPLErr Decompress(buf_mgr &dst, buf_mgr &src) override;
@@ -670,8 +693,8 @@ class Raw_Band : public GDALMRFRasterBand {
     friend class GDALMRFDataset;
 public:
     Raw_Band(GDALMRFDataset *pDS, const ILImage &image, int b, int level) :
-        GDALMRFRasterBand(pDS, image, b, int(level)) {};
-    virtual ~Raw_Band() {};
+        GDALMRFRasterBand(pDS, image, b, int(level)) {}
+    virtual ~Raw_Band() {}
 protected:
     virtual CPLErr Decompress(buf_mgr &dst, buf_mgr &src) override;
     virtual CPLErr Compress(buf_mgr &dst, buf_mgr &src) override;
@@ -715,7 +738,7 @@ protected:
  * Stand alone definition of a derived band, used in access to a specific level in an MRF
  *
  */
-class GDALMRFLRasterBand : public GDALPamRasterBand {
+class GDALMRFLRasterBand final: public GDALPamRasterBand {
 public:
     explicit GDALMRFLRasterBand(GDALMRFRasterBand *b) {
         pBand = b;
@@ -749,7 +772,7 @@ public:
 
 protected:
     virtual int GetOverviewCount() override { return 0; }
-    virtual GDALRasterBand *GetOverview(int ) override { return NULL; }
+    virtual GDALRasterBand *GetOverview(int ) override { return nullptr; }
 
     GDALMRFRasterBand *pBand;
 };
