@@ -34,9 +34,9 @@
 #include <algorithm>
 #include <vector>
 
-CPL_CVSID("$Id: xyzdataset.cpp 39396 2017-06-30 09:23:09Z rouault $");
+CPL_CVSID("$Id: xyzdataset.cpp feba3e02b56cfd80e5e98cfd3a822fad3bf521ba 2018-03-23 13:56:23 +0100 Even Rouault $")
 
-static const double RELATIVE_ERROR = 1e-3;
+constexpr double RELATIVE_ERROR = 1e-3;
 
 /************************************************************************/
 /* ==================================================================== */
@@ -66,7 +66,8 @@ class XYZDataset : public GDALPamDataset
     double      dfMaxZ;
     bool        bEOF;
 
-    static int          IdentifyEx( GDALOpenInfo *, int&, int& nCommentLineCount );
+    static int          IdentifyEx( GDALOpenInfo *, int&, int& nCommentLineCount,
+                                    int& nXIndex, int& nYIndex, int& nZIndex );
 
   public:
                  XYZDataset();
@@ -98,9 +99,9 @@ class XYZRasterBand : public GDALPamRasterBand
                 XYZRasterBand( XYZDataset *, int, GDALDataType );
 
     virtual CPLErr IReadBlock( int, int, void * ) override;
-    virtual double GetMinimum( int *pbSuccess = NULL ) override;
-    virtual double GetMaximum( int *pbSuccess = NULL ) override;
-    virtual double GetNoDataValue( int *pbSuccess = NULL ) override;
+    virtual double GetMinimum( int *pbSuccess = nullptr ) override;
+    virtual double GetMaximum( int *pbSuccess = nullptr ) override;
+    virtual double GetNoDataValue( int *pbSuccess = nullptr ) override;
 };
 
 /************************************************************************/
@@ -129,7 +130,7 @@ CPLErr XYZRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
 {
     XYZDataset *poGDS = reinterpret_cast<XYZDataset *>( poDS );
 
-    if (poGDS->fp == NULL)
+    if (poGDS->fp == nullptr)
         return CE_Failure;
 
     if( pImage )
@@ -155,7 +156,7 @@ CPLErr XYZRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
 
         for(int i=0;i<poGDS->nCommentLineCount;i++)
         {
-            if( CPLReadLine2L(poGDS->fp, 100, NULL) == NULL )
+            if( CPLReadLine2L(poGDS->fp, 100, nullptr) == nullptr )
             {
                 poGDS->bEOF = true;
                 return CE_Failure;
@@ -165,8 +166,8 @@ CPLErr XYZRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
 
         if (poGDS->bHasHeaderLine)
         {
-            const char* pszLine = CPLReadLine2L(poGDS->fp, 100, NULL);
-            if (pszLine == NULL)
+            const char* pszLine = CPLReadLine2L(poGDS->fp, 100, nullptr);
+            if (pszLine == nullptr)
             {
                 poGDS->bEOF = true;
                 return CE_Failure;
@@ -182,7 +183,7 @@ CPLErr XYZRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
             nLastYOff = -1;
             for( int iY = 0; iY < nBlockYOff; iY++ )
             {
-                if( IReadBlock(0, iY, NULL) != CE_None )
+                if( IReadBlock(0, iY, nullptr) != CE_None )
                     return CE_Failure;
             }
         }
@@ -194,7 +195,7 @@ CPLErr XYZRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
             }
             for( int iY = nLastYOff + 1; iY < nBlockYOff; iY++ )
             {
-                if( IReadBlock(0, iY, NULL) != CE_None )
+                if( IReadBlock(0, iY, nullptr) != CE_None )
                     return CE_Failure;
             }
         }
@@ -207,8 +208,8 @@ CPLErr XYZRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
         }
         while(poGDS->nDataLineNum < nLineInFile)
         {
-            const char* pszLine = CPLReadLine2L(poGDS->fp, 100, NULL);
-            if (pszLine == NULL)
+            const char* pszLine = CPLReadLine2L(poGDS->fp, 100, nullptr);
+            if (pszLine == nullptr)
             {
                 poGDS->bEOF = true;
                 return CE_Failure;
@@ -259,8 +260,8 @@ CPLErr XYZRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff,
         do
         {
             const vsi_l_offset nOffsetBefore = VSIFTellL(poGDS->fp);
-            const char* pszLine = CPLReadLine2L(poGDS->fp, 100, NULL);
-            if (pszLine == NULL)
+            const char* pszLine = CPLReadLine2L(poGDS->fp, 100, nullptr);
+            if (pszLine == nullptr)
             {
                 poGDS->bEOF = true;
                 if( poGDS->bSameNumberOfValuesPerLine )
@@ -462,7 +463,7 @@ double XYZRasterBand::GetNoDataValue(int *pbSuccess)
 /************************************************************************/
 
 XYZDataset::XYZDataset() :
-    fp(NULL),
+    fp(nullptr),
     bHasHeaderLine(FALSE),
     nCommentLineCount(0),
     chDecimalSep('.'),
@@ -504,7 +505,11 @@ XYZDataset::~XYZDataset()
 int XYZDataset::Identify( GDALOpenInfo * poOpenInfo )
 {
     int bHasHeaderLine, nCommentLineCount;
-    return IdentifyEx(poOpenInfo, bHasHeaderLine, nCommentLineCount);
+    int nXIndex;
+    int nYIndex;
+    int nZIndex;
+    return IdentifyEx(poOpenInfo, bHasHeaderLine, nCommentLineCount,
+                      nXIndex, nYIndex, nZIndex);
 }
 
 /************************************************************************/
@@ -513,7 +518,8 @@ int XYZDataset::Identify( GDALOpenInfo * poOpenInfo )
 
 int XYZDataset::IdentifyEx( GDALOpenInfo * poOpenInfo,
                             int& bHasHeaderLine,
-                            int& nCommentLineCount)
+                            int& nCommentLineCount,
+                            int& nXIndex, int& nYIndex, int& nZIndex)
 
 {
     bHasHeaderLine = FALSE;
@@ -521,7 +527,7 @@ int XYZDataset::IdentifyEx( GDALOpenInfo * poOpenInfo,
 
     CPLString osFilename(poOpenInfo->pszFilename);
 
-    GDALOpenInfo* poOpenInfoToDelete = NULL;
+    GDALOpenInfo* poOpenInfoToDelete = nullptr;
     /*  GZipped .xyz files are common, so automagically open them */
     /*  if the /vsigzip/ has not been explicitly passed */
     if (strlen(poOpenInfo->pszFilename) > 6 &&
@@ -547,6 +553,13 @@ int XYZDataset::IdentifyEx( GDALOpenInfo * poOpenInfo,
     const char* pszData
         = reinterpret_cast<const char *>( poOpenInfo->pabyHeader );
 
+    if( poOpenInfo->nHeaderBytes >= 4 && STARTS_WITH(pszData, "DSAA") )
+    {
+        // Do not match GSAG datasets
+        delete poOpenInfoToDelete;
+        return FALSE;
+    }
+
     /* Skip comments line at the beginning such as in */
     /* http://pubs.usgs.gov/of/2003/ofr-03-230/DATA/NSLCU.XYZ */
     int i = 0;
@@ -573,6 +586,7 @@ int XYZDataset::IdentifyEx( GDALOpenInfo * poOpenInfo,
         }
     }
 
+    int iStartLine = i;
     for( ; i < poOpenInfo->nHeaderBytes; i++ )
     {
         const char ch = pszData[i];
@@ -594,6 +608,40 @@ int XYZDataset::IdentifyEx( GDALOpenInfo * poOpenInfo,
             return FALSE;
         }
     }
+
+    nXIndex = -1;
+    nYIndex = -1;
+    nZIndex = -1;
+    if( bHasHeaderLine )
+    {
+        CPLString osHeaderLine;
+        osHeaderLine.assign(pszData + iStartLine, i - iStartLine);
+        char** papszTokens = CSLTokenizeString2( osHeaderLine, " ,\t;",
+                                                 CSLT_HONOURSTRINGS );
+        int nTokens = CSLCount(papszTokens);
+        for( i = 0; i < nTokens; i++ )
+        {
+            if (EQUAL(papszTokens[i], "x") ||
+                STARTS_WITH_CI(papszTokens[i], "lon") ||
+                STARTS_WITH_CI(papszTokens[i], "east"))
+                nXIndex = i;
+            else if (EQUAL(papszTokens[i], "y") ||
+                     STARTS_WITH_CI(papszTokens[i], "lat") ||
+                     STARTS_WITH_CI(papszTokens[i], "north"))
+                nYIndex = i;
+            else if (EQUAL(papszTokens[i], "z") ||
+                     STARTS_WITH_CI(papszTokens[i], "alt") ||
+                     EQUAL(papszTokens[i], "height"))
+                nZIndex = i;
+        }
+        CSLDestroy(papszTokens);
+        if( nXIndex >= 0 && nYIndex >= 0 && nZIndex >= 0)
+        {
+            delete poOpenInfoToDelete;
+            return TRUE;
+        }
+    }
+
     bool bHasFoundNewLine = false;
     bool bPrevWasSep = true;
     int nCols = 0;
@@ -649,8 +697,12 @@ GDALDataset *XYZDataset::Open( GDALOpenInfo * poOpenInfo )
     int         bHasHeaderLine;
     int         nCommentLineCount = 0;
 
-    if (!IdentifyEx(poOpenInfo, bHasHeaderLine, nCommentLineCount))
-        return NULL;
+    int nXIndex = -1;
+    int nYIndex = -1;
+    int nZIndex = -1;
+    if (!IdentifyEx(poOpenInfo, bHasHeaderLine, nCommentLineCount,
+                    nXIndex, nYIndex, nZIndex))
+        return nullptr;
 
     CPLString osFilename(poOpenInfo->pszFilename);
 
@@ -668,8 +720,8 @@ GDALDataset *XYZDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Find dataset characteristics                                    */
 /* -------------------------------------------------------------------- */
     VSILFILE* fp = VSIFOpenL(osFilename.c_str(), "rb");
-    if (fp == NULL)
-        return NULL;
+    if (fp == nullptr)
+        return nullptr;
 
     /* For better performance of CPLReadLine2L() we create a buffered reader */
     /* (except for /vsigzip/ since it has one internally) */
@@ -678,17 +730,14 @@ GDALDataset *XYZDataset::Open( GDALOpenInfo * poOpenInfo )
             VSICreateBufferedReaderHandle(
                 reinterpret_cast<VSIVirtualHandle *>( fp ) ) );
 
-    int nXIndex = -1;
-    int nYIndex = -1;
-    int nZIndex = -1;
     int nMinTokens = 0;
 
     for( int i = 0; i < nCommentLineCount; i++ )
     {
-        if( CPLReadLine2L(fp, 100, NULL) == NULL )
+        if( CPLReadLine2L(fp, 100, nullptr) == nullptr )
         {
             VSIFCloseL(fp);
-            return NULL;
+            return nullptr;
         }
     }
 
@@ -697,41 +746,12 @@ GDALDataset *XYZDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     if (bHasHeaderLine)
     {
-        const char* pszLine = CPLReadLine2L(fp, 100, NULL);
-        if (pszLine == NULL)
+        const char* pszLine = CPLReadLine2L(fp, 100, nullptr);
+        if (pszLine == nullptr)
         {
             VSIFCloseL(fp);
-            return NULL;
+            return nullptr;
         }
-        char** papszTokens = CSLTokenizeString2( pszLine, " ,\t;",
-                                                 CSLT_HONOURSTRINGS );
-        int nTokens = CSLCount(papszTokens);
-        if (nTokens < 3)
-        {
-            CPLError(CE_Failure, CPLE_AppDefined,
-                     "At line %d, found %d tokens. Expected 3 at least",
-                      1, nTokens);
-            CSLDestroy(papszTokens);
-            VSIFCloseL(fp);
-            return NULL;
-        }
-        for( int i = 0; i < nTokens; i++ )
-        {
-            if (EQUAL(papszTokens[i], "x") ||
-                STARTS_WITH_CI(papszTokens[i], "lon") ||
-                STARTS_WITH_CI(papszTokens[i], "east"))
-                nXIndex = i;
-            else if (EQUAL(papszTokens[i], "y") ||
-                     STARTS_WITH_CI(papszTokens[i], "lat") ||
-                     STARTS_WITH_CI(papszTokens[i], "north"))
-                nYIndex = i;
-            else if (EQUAL(papszTokens[i], "z") ||
-                     STARTS_WITH_CI(papszTokens[i], "alt") ||
-                     EQUAL(papszTokens[i], "height"))
-                nZIndex = i;
-        }
-        CSLDestroy(papszTokens);
-        papszTokens = NULL;
         if (nXIndex < 0 || nYIndex < 0 || nZIndex < 0)
         {
             CPLError(CE_Warning, CPLE_AppDefined,
@@ -777,7 +797,7 @@ GDALDataset *XYZDataset::Open( GDALOpenInfo * poOpenInfo )
     const char* pszLine;
     GIntBig nCountStepX = 0;
     GIntBig nCountStepY = 0;
-    while((pszLine = CPLReadLine2L(fp, 100, NULL)) != NULL)
+    while((pszLine = CPLReadLine2L(fp, 100, nullptr)) != nullptr)
     {
         nLineNum ++;
 
@@ -915,7 +935,7 @@ GDALDataset *XYZDataset::Open( GDALOpenInfo * poOpenInfo )
                      "At line " CPL_FRMT_GIB ", found %d tokens. Expected %d at least",
                       nLineNum, nCol, nMinTokens);
             VSIFCloseL(fp);
-            return NULL;
+            return nullptr;
         }
         if( nUsefulColsFound != 3 )
         {
@@ -923,7 +943,7 @@ GDALDataset *XYZDataset::Open( GDALOpenInfo * poOpenInfo )
                      "At line " CPL_FRMT_GIB ", did not find X, Y and/or Z values",
                       nLineNum);
             VSIFCloseL(fp);
-            return NULL;
+            return nullptr;
         }
 
         if (nDataLineNum == 1)
@@ -945,7 +965,7 @@ GDALDataset *XYZDataset::Open( GDALOpenInfo * poOpenInfo )
                          "Ungridded dataset: At line " CPL_FRMT_GIB ", X spacing was %f. Expected >0 value",
                          nLineNum, dfStepX);
                     VSIFCloseL(fp);
-                    return NULL;
+                    return nullptr;
                 }
                 if( std::find(adfStepX.begin(), adfStepX.end(), dfStepX) == adfStepX.end() )
                 {
@@ -1008,7 +1028,7 @@ GDALDataset *XYZDataset::Open( GDALOpenInfo * poOpenInfo )
                             CPLError(CE_Failure, CPLE_AppDefined,
                                 "Ungridded dataset: too many stepX values");
                             VSIFCloseL(fp);
-                            return NULL;
+                            return nullptr;
                         }
                     }
                 }
@@ -1024,7 +1044,7 @@ GDALDataset *XYZDataset::Open( GDALOpenInfo * poOpenInfo )
                          "Ungridded dataset: At line " CPL_FRMT_GIB ", change of Y direction",
                          nLineNum);
                     VSIFCloseL(fp);
-                    return NULL;
+                    return nullptr;
                 }
                 if( bNewStepYSign < 0 ) dfStepY = -dfStepY;
                 nCountStepY ++;
@@ -1038,7 +1058,7 @@ GDALDataset *XYZDataset::Open( GDALOpenInfo * poOpenInfo )
                     CPLError(CE_Failure, CPLE_AppDefined,
                         "Ungridded dataset: At line " CPL_FRMT_GIB ", too many stepY values", nLineNum);
                     VSIFCloseL(fp);
-                    return NULL;
+                    return nullptr;
                 }
                 else
                 {
@@ -1061,23 +1081,25 @@ GDALDataset *XYZDataset::Open( GDALOpenInfo * poOpenInfo )
     {
         CPLError(CE_Failure, CPLE_AppDefined, "Couldn't determine X spacing");
         VSIFCloseL(fp);
-        return NULL;
+        return nullptr;
     }
 
     if (adfStepY.size() != 1)
     {
         CPLError(CE_Failure, CPLE_AppDefined, "Couldn't determine Y spacing");
         VSIFCloseL(fp);
-        return NULL;
+        return nullptr;
     }
 
     const double dfXSize = 1 + ((dfMaxX - dfMinX) / adfStepX[0] + 0.5);
     const double dfYSize = 1 + ((dfMaxY - dfMinY) / adfStepY[0] + 0.5);
-    if( dfXSize <= 0 || dfXSize > INT_MAX || dfYSize <= 0 || dfYSize > INT_MAX )
+    // Test written such as to detect NaN values
+    if( !(dfXSize > 0 && dfXSize < INT_MAX) ||
+        !(dfYSize > 0 && dfYSize < INT_MAX ) )
     {
         CPLError(CE_Failure, CPLE_AppDefined, "Invalid dimensions");
         VSIFCloseL(fp);
-        return NULL;
+        return nullptr;
     }
     const int nXSize = static_cast<int>(dfXSize);
     const int nYSize = static_cast<int>(dfYSize);
@@ -1100,7 +1122,7 @@ GDALDataset *XYZDataset::Open( GDALOpenInfo * poOpenInfo )
                   "The XYZ driver does not support update access to existing"
                   " datasets.\n" );
         VSIFCloseL(fp);
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -1133,7 +1155,7 @@ GDALDataset *XYZDataset::Open( GDALOpenInfo * poOpenInfo )
     if (!GDALCheckDatasetDimensions(poDS->nRasterXSize, poDS->nRasterYSize))
     {
         delete poDS;
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -1174,7 +1196,7 @@ GDALDataset* XYZDataset::CreateCopy( const char * pszFilename,
     {
         CPLError( CE_Failure, CPLE_NotSupported,
                   "XYZ driver does not support source dataset with zero band.\n");
-        return NULL;
+        return nullptr;
     }
 
     if (nBands != 1)
@@ -1182,11 +1204,11 @@ GDALDataset* XYZDataset::CreateCopy( const char * pszFilename,
         CPLError( (bStrict) ? CE_Failure : CE_Warning, CPLE_NotSupported,
                   "XYZ driver only uses the first band of the dataset.\n");
         if (bStrict)
-            return NULL;
+            return nullptr;
     }
 
-    if( pfnProgress && !pfnProgress( 0.0, NULL, pProgressData ) )
-        return NULL;
+    if( pfnProgress && !pfnProgress( 0.0, nullptr, pProgressData ) )
+        return nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      Get source dataset info                                         */
@@ -1200,7 +1222,7 @@ GDALDataset* XYZDataset::CreateCopy( const char * pszFilename,
     {
         CPLError( CE_Failure, CPLE_NotSupported,
                   "XYZ driver does not support CreateCopy() from skewed or rotated dataset.\n");
-        return NULL;
+        return nullptr;
     }
 
     const GDALDataType eSrcDT = poSrcDS->GetRasterBand(1)->GetRasterDataType();
@@ -1216,11 +1238,11 @@ GDALDataset* XYZDataset::CreateCopy( const char * pszFilename,
 /* -------------------------------------------------------------------- */
 
     VSILFILE* fp = VSIFOpenL(pszFilename, "wb");
-    if (fp == NULL)
+    if (fp == nullptr)
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "Cannot create %s", pszFilename );
-        return NULL;
+        return nullptr;
     }
 
 /* -------------------------------------------------------------------- */
@@ -1228,7 +1250,7 @@ GDALDataset* XYZDataset::CreateCopy( const char * pszFilename,
 /* -------------------------------------------------------------------- */
     const char* pszColSep =
             CSLFetchNameValue(papszOptions, "COLUMN_SEPARATOR");
-    if (pszColSep == NULL)
+    if (pszColSep == nullptr)
         pszColSep = " ";
     else if (EQUAL(pszColSep, "COMMA"))
         pszColSep = ",";
@@ -1246,7 +1268,7 @@ GDALDataset* XYZDataset::CreateCopy( const char * pszFilename,
 
     const char* pszAddHeaderLine =
             CSLFetchNameValue(papszOptions, "ADD_HEADER_LINE");
-    if (pszAddHeaderLine != NULL && CPLTestBool(pszAddHeaderLine))
+    if (pszAddHeaderLine != nullptr && CPLTestBool(pszAddHeaderLine))
     {
         VSIFPrintfL(fp, "X%sY%sZ\n", pszColSep, pszColSep);
     }
@@ -1262,7 +1284,7 @@ GDALDataset* XYZDataset::CreateCopy( const char * pszFilename,
         eErr = poSrcDS->GetRasterBand(1)->RasterIO(
                                             GF_Read, 0, j, nXSize, 1,
                                             pLineBuffer, nXSize, 1,
-                                            eReqDT, 0, 0, NULL);
+                                            eReqDT, 0, 0, nullptr);
         if (eErr != CE_None)
             break;
         const double dfY = adfGeoTransform[3] + (j + 0.5) * adfGeoTransform[5];
@@ -1296,7 +1318,7 @@ GDALDataset* XYZDataset::CreateCopy( const char * pszFilename,
             }
         }
         if ( pfnProgress
-             && !pfnProgress( (j+1) * 1.0 / nYSize, NULL, pProgressData ) )
+             && !pfnProgress( (j+1) * 1.0 / nYSize, nullptr, pProgressData ) )
         {
             eErr = CE_Failure;
             break;
@@ -1306,7 +1328,7 @@ GDALDataset* XYZDataset::CreateCopy( const char * pszFilename,
     VSIFCloseL(fp);
 
     if (eErr != CE_None)
-        return NULL;
+        return nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      We don't want to call GDALOpen() since it will be expensive,    */
@@ -1353,7 +1375,7 @@ CPLErr XYZDataset::GetGeoTransform( double * padfTransform )
 void GDALRegister_XYZ()
 
 {
-    if( GDALGetDriverByName( "XYZ" ) != NULL )
+    if( GDALGetDriverByName( "XYZ" ) != nullptr )
       return;
 
     GDALDriver *poDriver = new GDALDriver();
