@@ -31,7 +31,7 @@
 #include "ogr_pgdump.h"
 #include "ogrgeojsonreader.h"
 
-CPL_CVSID("$Id: ogrcartotablelayer.cpp 41751 2018-03-12 15:25:43Z rouault $");
+CPL_CVSID("$Id: ogrcartotablelayer.cpp 6574497e5ddfd7c08c094a76756a0ef477cef6a1 2018-04-04 22:15:20 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                    OGRCARTOEscapeIdentifier( )                     */
@@ -113,7 +113,7 @@ OGRCARTOTableLayer::~OGRCARTOTableLayer()
 
 OGRFeatureDefn * OGRCARTOTableLayer::GetLayerDefnInternal(CPL_UNUSED json_object* poObjIn)
 {
-    if( poFeatureDefn != NULL )
+    if( poFeatureDefn != nullptr )
         return poFeatureDefn;
 
     CPLString osCommand;
@@ -160,7 +160,7 @@ OGRFeatureDefn * OGRCARTOTableLayer::GetLayerDefnInternal(CPL_UNUSED json_object
         if( !poDS->IsAuthenticatedConnection() && poDS->HasOGRMetadataFunction() < 0 )
         {
             CPLPopErrorHandler();
-            if( poLyr == NULL )
+            if( poLyr == nullptr )
             {
                 CPLDebug("CARTO", "ogr_table_metadata(text, text) not available");
                 CPLErrorReset();
@@ -169,16 +169,16 @@ OGRFeatureDefn * OGRCARTOTableLayer::GetLayerDefnInternal(CPL_UNUSED json_object
             {
                 CPLDebug("CARTO", "ogr_table_metadata(text, text) has unexpected column count");
                 poDS->ReleaseResultSet(poLyr);
-                poLyr = NULL;
+                poLyr = nullptr;
             }
-            poDS->SetOGRMetadataFunction(poLyr != NULL);
+            poDS->SetOGRMetadataFunction(poLyr != nullptr);
         }
         if( poLyr )
         {
             OGRFeature* poFeat;
-            while( (poFeat = poLyr->GetNextFeature()) != NULL )
+            while( (poFeat = poLyr->GetNextFeature()) != nullptr )
             {
-                if( poFeatureDefn == NULL )
+                if( poFeatureDefn == nullptr )
                 {
                     // We could do that outside of the while() loop, but
                     // by doing that here, we are somewhat robust to
@@ -197,7 +197,7 @@ OGRFeatureDefn * OGRCARTOTableLayer::GetLayerDefnInternal(CPL_UNUSED json_object
                 int bIsPrimary = poFeat->GetFieldAsInteger("indisprimary");
                 int iDefaultExpr = poLyr->GetLayerDefn()->GetFieldIndex("defaultexpr");
                 const char* pszDefault = (iDefaultExpr >= 0 && poFeat->IsFieldSetAndNotNull(iDefaultExpr)) ?
-                            poFeat->GetFieldAsString(iDefaultExpr) : NULL;
+                            poFeat->GetFieldAsString(iDefaultExpr) : nullptr;
 
                 if( bIsPrimary &&
                     (EQUAL(pszType, "int2") ||
@@ -223,7 +223,7 @@ OGRFeatureDefn * OGRCARTOTableLayer::GetLayerDefnInternal(CPL_UNUSED json_object
                         const char* pszGeomType = poFeat->GetFieldAsString("geomtyp");
                         const char* pszSRText = (poFeat->IsFieldSetAndNotNull(
                             poLyr->GetLayerDefn()->GetFieldIndex("srtext"))) ?
-                                    poFeat->GetFieldAsString("srtext") : NULL;
+                                    poFeat->GetFieldAsString("srtext") : nullptr;
                         OGRwkbGeometryType eType = OGRFromOGCGeomType(pszGeomType);
                         if( nDim == 3 )
                             eType = wkbSetZ(eType);
@@ -231,17 +231,16 @@ OGRFeatureDefn * OGRCARTOTableLayer::GetLayerDefnInternal(CPL_UNUSED json_object
                             new OGRCartoGeomFieldDefn(pszAttname, eType);
                         if( bNotNull )
                             poFieldDefn->SetNullable(FALSE);
-                        OGRSpatialReference* l_poSRS = NULL;
-                        if( pszSRText != NULL )
+                        OGRSpatialReference* l_poSRS = nullptr;
+                        if( pszSRText != nullptr )
                         {
                             l_poSRS = new OGRSpatialReference();
-                            char* pszTmp = (char* )pszSRText;
-                            if( l_poSRS->importFromWkt(&pszTmp) != OGRERR_NONE )
+                            if( l_poSRS->importFromWkt(pszSRText) != OGRERR_NONE )
                             {
                                 delete l_poSRS;
-                                l_poSRS = NULL;
+                                l_poSRS = nullptr;
                             }
-                            if( l_poSRS != NULL )
+                            if( l_poSRS != nullptr )
                             {
                                 poFieldDefn->SetSpatialRef(l_poSRS);
                                 l_poSRS->Release();
@@ -269,10 +268,10 @@ OGRFeatureDefn * OGRCARTOTableLayer::GetLayerDefnInternal(CPL_UNUSED json_object
         }
     }
 
-    if( poFeatureDefn == NULL )
+    if( poFeatureDefn == nullptr )
     {
         osBaseSQL.Printf("SELECT * FROM %s", OGRCARTOEscapeIdentifier(osName).c_str());
-        EstablishLayerDefn(osName, NULL);
+        EstablishLayerDefn(osName, nullptr);
         osBaseSQL = "";
     }
 
@@ -336,9 +335,9 @@ json_object* OGRCARTOTableLayer::FetchNewFeatures()
 OGRFeature  *OGRCARTOTableLayer::GetNextRawFeature()
 {
     if( bDeferredCreation && RunDeferredCreationIfNecessary() != OGRERR_NONE )
-        return NULL;
+        return nullptr;
     if( FlushDeferredInsert() != OGRERR_NONE )
-        return NULL;
+        return nullptr;
     return OGRCARTOLayer::GetNextRawFeature();
 }
 
@@ -351,7 +350,7 @@ OGRErr OGRCARTOTableLayer::SetAttributeFilter( const char *pszQuery )
 {
     GetLayerDefn();
 
-    if( pszQuery == NULL )
+    if( pszQuery == nullptr )
         osQuery = "";
     else
     {
@@ -415,7 +414,7 @@ void OGRCARTOTableLayer::RunDeferredCartofy()
                                 OGRCARTOEscapeLiteral(osName).c_str());
 
         json_object* poObj = poDS->RunSQL(osSQL);
-        if( poObj != NULL )
+        if( poObj != nullptr )
             json_object_put(poObj);
     }
 }
@@ -439,7 +438,7 @@ OGRErr OGRCARTOTableLayer::FlushDeferredInsert(bool bReset)
         osDeferredInsertSQL += "COMMIT;";
 
         json_object* poObj = poDS->RunSQL(osDeferredInsertSQL);
-        if( poObj != NULL )
+        if( poObj != nullptr )
         {
             json_object_put(poObj);
         }
@@ -502,14 +501,14 @@ OGRErr OGRCARTOTableLayer::CreateField( OGRFieldDefn *poFieldIn,
                     OGRPGCommonLayerGetType(oField, false, true).c_str() );
         if( !oField.IsNullable() )
             osSQL += " NOT NULL";
-        if( oField.GetDefault() != NULL && !oField.IsDefaultDriverSpecific() )
+        if( oField.GetDefault() != nullptr && !oField.IsDefaultDriverSpecific() )
         {
             osSQL += " DEFAULT ";
             osSQL += OGRPGCommonLayerGetPGDefault(&oField);
         }
 
         json_object* poObj = poDS->RunSQL(osSQL);
-        if( poObj == NULL )
+        if( poObj == nullptr )
             return OGRERR_FAILURE;
         json_object_put(poObj);
     }
@@ -556,7 +555,7 @@ OGRErr OGRCARTOTableLayer::DeleteField( int iField )
                   OGRCARTOEscapeIdentifier(poFeatureDefn->GetFieldDefn(iField)->GetNameRef()).c_str() );
 
     json_object* poObj = poDS->RunSQL(osSQL);
-    if( poObj == NULL )
+    if( poObj == nullptr )
         return OGRERR_FAILURE;
     json_object_put(poObj);
 
@@ -599,16 +598,16 @@ OGRErr OGRCARTOTableLayer::ICreateFeature( OGRFeature *poFeature )
                      OGRCARTOEscapeLiteral(osFIDColName).c_str());
         json_object* poObj = poDS->RunSQL(osSQL);
         json_object* poRowObj = OGRCARTOGetSingleRow(poObj);
-        if( poRowObj != NULL )
+        if( poRowObj != nullptr )
         {
             json_object* poSeqName = CPL_json_object_object_get(poRowObj, "seq_name");
-            if( poSeqName != NULL && json_object_get_type(poSeqName) == json_type_string )
+            if( poSeqName != nullptr && json_object_get_type(poSeqName) == json_type_string )
             {
                 osSeqName = json_object_get_string(poSeqName);
             }
         }
 
-        if( poObj != NULL )
+        if( poObj != nullptr )
             json_object_put(poObj);
 
         if( !osSeqName.empty() )
@@ -618,17 +617,17 @@ OGRErr OGRCARTOTableLayer::ICreateFeature( OGRFeature *poFeature )
 
             poObj = poDS->RunSQL(osSQL);
             poRowObj = OGRCARTOGetSingleRow(poObj);
-            if( poRowObj != NULL )
+            if( poRowObj != nullptr )
             {
                 json_object* poID = CPL_json_object_object_get(poRowObj, "nextid");
-                if( poID != NULL && json_object_get_type(poID) == json_type_int )
+                if( poID != nullptr && json_object_get_type(poID) == json_type_int )
                 {
                     m_nNextFIDWrite = json_object_get_int64(poID);
                     bHasJustGotNextFID = true;
                 }
             }
 
-            if( poObj != NULL )
+            if( poObj != nullptr )
                 json_object_put(poObj);
         }
     }
@@ -663,7 +662,7 @@ OGRErr OGRCARTOTableLayer::ICreateFeature( OGRFeature *poFeature )
             eDeferredInsertState = INSERT_MULTIPLE_FEATURE;
             for( int i = 0; i < poFeatureDefn->GetFieldCount(); i++ )
             {
-                if( poFeatureDefn->GetFieldDefn(i)->GetDefault() != NULL )
+                if( poFeatureDefn->GetFieldDefn(i)->GetDefault() != nullptr )
                     eDeferredInsertState = INSERT_SINGLE_FEATURE;
             }
         }
@@ -693,7 +692,7 @@ OGRErr OGRCARTOTableLayer::ICreateFeature( OGRFeature *poFeature )
         for( int i = 0; i < poFeatureDefn->GetGeomFieldCount(); i++ )
         {
             if( eDeferredInsertState != INSERT_MULTIPLE_FEATURE &&
-                poFeature->GetGeomFieldRef(i) == NULL )
+                poFeature->GetGeomFieldRef(i) == nullptr )
                 continue;
 
             if( bMustComma )
@@ -778,7 +777,7 @@ OGRErr OGRCARTOTableLayer::ICreateFeature( OGRFeature *poFeature )
         for( int i = 0; i < poFeatureDefn->GetGeomFieldCount(); i++ )
         {
             OGRGeometry* poGeom = poFeature->GetGeomFieldRef(i);
-            if( poGeom == NULL )
+            if( poGeom == nullptr )
             {
                 if( eDeferredInsertState == INSERT_MULTIPLE_FEATURE )
                 {
@@ -887,20 +886,20 @@ OGRErr OGRCARTOTableLayer::ICreateFeature( OGRFeature *poFeature )
 
         json_object* poObj = poDS->RunSQL(osSQL);
         json_object* poRowObj = OGRCARTOGetSingleRow(poObj);
-        if( poRowObj == NULL )
+        if( poRowObj == nullptr )
         {
-            if( poObj != NULL )
+            if( poObj != nullptr )
                 json_object_put(poObj);
             return OGRERR_FAILURE;
         }
 
         json_object* poID = CPL_json_object_object_get(poRowObj, osFIDColName);
-        if( poID != NULL && json_object_get_type(poID) == json_type_int )
+        if( poID != nullptr && json_object_get_type(poID) == json_type_int )
         {
             poFeature->SetFID(json_object_get_int64(poID));
         }
 
-        if( poObj != NULL )
+        if( poObj != nullptr )
             json_object_put(poObj);
 
         return OGRERR_NONE;
@@ -909,10 +908,10 @@ OGRErr OGRCARTOTableLayer::ICreateFeature( OGRFeature *poFeature )
     {
         OGRErr eRet = OGRERR_FAILURE;
         json_object* poObj = poDS->RunSQL(osSQL);
-        if( poObj != NULL )
+        if( poObj != nullptr )
         {
             json_object* poTotalRows = CPL_json_object_object_get(poObj, "total_rows");
-            if( poTotalRows != NULL && json_object_get_type(poTotalRows) == json_type_int )
+            if( poTotalRows != nullptr && json_object_get_type(poTotalRows) == json_type_int )
             {
                 int nTotalRows = json_object_get_int(poTotalRows);
                 if( nTotalRows == 1 )
@@ -1005,7 +1004,7 @@ OGRErr OGRCARTOTableLayer::ISetFeature( OGRFeature *poFeature )
         osSQL += " = ";
 
         OGRGeometry* poGeom = poFeature->GetGeomFieldRef(i);
-        if( poGeom == NULL )
+        if( poGeom == nullptr )
         {
             osSQL += "NULL";
         }
@@ -1035,10 +1034,10 @@ OGRErr OGRCARTOTableLayer::ISetFeature( OGRFeature *poFeature )
 
     OGRErr eRet = OGRERR_FAILURE;
     json_object* poObj = poDS->RunSQL(osSQL);
-    if( poObj != NULL )
+    if( poObj != nullptr )
     {
         json_object* poTotalRows = CPL_json_object_object_get(poObj, "total_rows");
-        if( poTotalRows != NULL && json_object_get_type(poTotalRows) == json_type_int )
+        if( poTotalRows != nullptr && json_object_get_type(poTotalRows) == json_type_int )
         {
             int nTotalRows = json_object_get_int(poTotalRows);
             if( nTotalRows > 0 )
@@ -1087,10 +1086,10 @@ OGRErr OGRCARTOTableLayer::DeleteFeature( GIntBig nFID )
 
     OGRErr eRet = OGRERR_FAILURE;
     json_object* poObj = poDS->RunSQL(osSQL);
-    if( poObj != NULL )
+    if( poObj != nullptr )
     {
         json_object* poTotalRows = CPL_json_object_object_get(poObj, "total_rows");
-        if( poTotalRows != NULL && json_object_get_type(poTotalRows) == json_type_int )
+        if( poTotalRows != nullptr && json_object_get_type(poTotalRows) == json_type_int )
         {
             int nTotalRows = json_object_get_int(poTotalRows);
             if( nTotalRows > 0 )
@@ -1135,7 +1134,7 @@ void OGRCARTOTableLayer::BuildWhere()
 {
     osWHERE = "";
 
-    if( m_poFilterGeom != NULL &&
+    if( m_poFilterGeom != nullptr &&
         m_iGeomFieldFilter >= 0 &&
         m_iGeomFieldFilter < poFeatureDefn->GetGeomFieldCount() )
     {
@@ -1150,10 +1149,10 @@ void OGRCARTOTableLayer::BuildWhere()
         char* pszComma;
 
         CPLsnprintf(szBox3D_1, sizeof(szBox3D_1), "%.18g %.18g", sEnvelope.MinX, sEnvelope.MinY);
-        while((pszComma = strchr(szBox3D_1, ',')) != NULL)
+        while((pszComma = strchr(szBox3D_1, ',')) != nullptr)
             *pszComma = '.';
         CPLsnprintf(szBox3D_2, sizeof(szBox3D_2), "%.18g %.18g", sEnvelope.MaxX, sEnvelope.MaxY);
-        while((pszComma = strchr(szBox3D_2, ',')) != NULL)
+        while((pszComma = strchr(szBox3D_2, ',')) != nullptr)
             *pszComma = '.';
         osWHERE.Printf("(%s && 'BOX3D(%s, %s)'::box3d)",
                        OGRCARTOEscapeIdentifier(osGeomColumn).c_str(),
@@ -1186,9 +1185,9 @@ OGRFeature* OGRCARTOTableLayer::GetFeature( GIntBig nFeatureId )
 {
 
     if( bDeferredCreation && RunDeferredCreationIfNecessary() != OGRERR_NONE )
-        return NULL;
+        return nullptr;
     if( FlushDeferredInsert() != OGRERR_NONE )
-        return NULL;
+        return nullptr;
 
     GetLayerDefn();
 
@@ -1203,9 +1202,9 @@ OGRFeature* OGRCARTOTableLayer::GetFeature( GIntBig nFeatureId )
 
     json_object* poObj = poDS->RunSQL(osSQL);
     json_object* poRowObj = OGRCARTOGetSingleRow(poObj);
-    if( poRowObj == NULL )
+    if( poRowObj == nullptr )
     {
-        if( poObj != NULL )
+        if( poObj != nullptr )
             json_object_put(poObj);
         return OGRCARTOLayer::GetFeature(nFeatureId);
     }
@@ -1240,15 +1239,15 @@ GIntBig OGRCARTOTableLayer::GetFeatureCount(int bForce)
 
     json_object* poObj = poDS->RunSQL(osSQL);
     json_object* poRowObj = OGRCARTOGetSingleRow(poObj);
-    if( poRowObj == NULL )
+    if( poRowObj == nullptr )
     {
-        if( poObj != NULL )
+        if( poObj != nullptr )
             json_object_put(poObj);
         return OGRCARTOLayer::GetFeatureCount(bForce);
     }
 
     json_object* poCount = CPL_json_object_object_get(poRowObj, "count");
-    if( poCount == NULL || json_object_get_type(poCount) != json_type_int )
+    if( poCount == nullptr || json_object_get_type(poCount) != json_type_int )
     {
         json_object_put(poObj);
         return OGRCARTOLayer::GetFeatureCount(bForce);
@@ -1298,10 +1297,10 @@ OGRErr OGRCARTOTableLayer::GetExtent( int iGeomField, OGREnvelope *psExtent, int
 
     json_object* poObj = poDS->RunSQL(osSQL);
     json_object* poRowObj = OGRCARTOGetSingleRow(poObj);
-    if( poRowObj != NULL )
+    if( poRowObj != nullptr )
     {
         json_object* poExtent = CPL_json_object_object_get(poRowObj, "st_extent");
-        if( poExtent != NULL && json_object_get_type(poExtent) == json_type_string )
+        if( poExtent != nullptr && json_object_get_type(poExtent) == json_type_string )
         {
             const char* pszBox = json_object_get_string(poExtent);
             const char * ptr, *ptrEndParenthesis;
@@ -1310,8 +1309,8 @@ OGRErr OGRCARTOTableLayer::GetExtent( int iGeomField, OGREnvelope *psExtent, int
             ptr = strchr(pszBox, '(');
             if (ptr)
                 ptr ++;
-            if (ptr == NULL ||
-                (ptrEndParenthesis = strchr(ptr, ')')) == NULL ||
+            if (ptr == nullptr ||
+                (ptrEndParenthesis = strchr(ptr, ')')) == nullptr ||
                 ptrEndParenthesis - ptr > (int)(sizeof(szVals) - 1))
             {
                 CPLError( CE_Failure, CPLE_IllegalArg,
@@ -1355,7 +1354,7 @@ OGRErr OGRCARTOTableLayer::GetExtent( int iGeomField, OGREnvelope *psExtent, int
         }
     }
 
-    if( poObj != NULL )
+    if( poObj != nullptr )
         json_object_put(poObj);
 
     if( iGeomField == 0 )
@@ -1404,7 +1403,7 @@ void OGRCARTOTableLayer::SetDeferredCreation( OGRwkbGeometryType eGType,
 {
     bDeferredCreation = true;
     m_nNextFIDWrite = 1;
-    CPLAssert(poFeatureDefn == NULL);
+    CPLAssert(poFeatureDefn == nullptr);
     bCartodbfy = bCartodbfyIn;
     poFeatureDefn = new OGRFeatureDefn(osName);
     poFeatureDefn->Reference();
@@ -1419,7 +1418,7 @@ void OGRCARTOTableLayer::SetDeferredCreation( OGRwkbGeometryType eGType,
             new OGRCartoGeomFieldDefn("the_geom", eGType);
         poFieldDefn->SetNullable(bGeomNullable);
         poFeatureDefn->AddGeomFieldDefn(poFieldDefn, FALSE);
-        if( poSRSIn != NULL )
+        if( poSRSIn != nullptr )
         {
             poFieldDefn->nSRID = poDS->FetchSRSId( poSRSIn );
             poFeatureDefn->GetGeomFieldDefn(
@@ -1476,7 +1475,7 @@ OGRErr OGRCARTOTableLayer::RunDeferredCreationIfNecessary()
             osSQL += OGRPGCommonLayerGetType(*poFieldDefn, false, true);
             if( !poFieldDefn->IsNullable() )
                 osSQL += " NOT NULL";
-            if( poFieldDefn->GetDefault() != NULL && !poFieldDefn->IsDefaultDriverSpecific() )
+            if( poFieldDefn->GetDefault() != nullptr && !poFieldDefn->IsDefaultDriverSpecific() )
             {
                 osSQL += " DEFAULT ";
                 osSQL += poFieldDefn->GetDefault();
@@ -1503,7 +1502,7 @@ OGRErr OGRCARTOTableLayer::RunDeferredCreationIfNecessary()
                         osFIDColName.c_str(), osSeqName.c_str());
 
     json_object* poObj = poDS->RunSQL(osSQL);
-    if( poObj == NULL )
+    if( poObj == nullptr )
         return OGRERR_FAILURE;
     json_object_put(poObj);
 
