@@ -37,7 +37,7 @@
 #include "cpl_conv.h"
 #include "cpl_error.h"
 
-CPL_CVSID("$Id: vfkdatablocksqlite.cpp 37472 2017-02-26 02:47:45Z goatbar $");
+CPL_CVSID("$Id: vfkdatablocksqlite.cpp 61f49c952d01d3971f455fa9aebac65eecee6e82 2018-04-11 13:49:00 +0200 Even Rouault $")
 
 /*!
   \brief Load geometry (point layers)
@@ -69,13 +69,15 @@ int VFKDataBlockSQLite::LoadGeometryPoint()
         /* read values */
         const double x = -1.0 * sqlite3_column_double(hStmt, 0); /* S-JTSK coordinate system expected */
         const double y = -1.0 * sqlite3_column_double(hStmt, 1);
-#ifdef DEBUG
         const GIntBig iFID = sqlite3_column_int64(hStmt, 2);
-#endif
         const int rowId = sqlite3_column_int(hStmt, 3);
 
-        VFKFeatureSQLite *poFeature = (VFKFeatureSQLite *) GetFeatureByIndex(rowId - 1);
-        CPLAssert(NULL != poFeature && poFeature->GetFID() == iFID);
+        VFKFeatureSQLite *poFeature = dynamic_cast<VFKFeatureSQLite *>(
+            GetFeatureByIndex(rowId - 1));
+        if( poFeature == nullptr || poFeature->GetFID() != iFID )
+        {
+            continue;
+        }
 
         /* create geometry */
         OGRPoint pt(x, y);
@@ -175,7 +177,7 @@ bool VFKDataBlockSQLite::SetGeometryLineString(VFKFeatureSQLite *poLine, OGRLine
     }
     else
     {
-        poLine->SetGeometry(NULL);
+        poLine->SetGeometry(nullptr);
     }
 
     /* update fid column */
@@ -184,7 +186,7 @@ bool VFKDataBlockSQLite::SetGeometryLineString(VFKFeatureSQLite *poLine, OGRLine
     /* store also geometry in DB */
     CPLAssert( !rowIdFeat.empty() );
     if( bValid && poReader->IsSpatial() &&
-        SaveGeometryToDB(bValid ? poLine->GetGeometry() : NULL,
+        SaveGeometryToDB(bValid ? poLine->GetGeometry() : nullptr,
                          rowIdFeat[0]) != OGRERR_FAILURE )
     {
         nGeometries++;
@@ -207,7 +209,7 @@ int VFKDataBlockSQLite::LoadGeometryLineStringSBP()
 
     VFKDataBlockSQLite *poDataBlockPoints =
         (VFKDataBlockSQLite *) m_poReader->GetDataBlock("SOBR");
-    if (NULL == poDataBlockPoints) {
+    if (nullptr == poDataBlockPoints) {
         CPLError(CE_Failure, CPLE_FileIO,
                  "Data block %s not found.\n", m_pszName);
         return nInvalid;
@@ -227,19 +229,19 @@ int VFKDataBlockSQLite::LoadGeometryLineStringSBP()
     bool bValid = true;
     int iIdx = 0;
 
-    VFKFeatureSQLite *poLine = NULL;
+    VFKFeatureSQLite *poLine = nullptr;
 
     for( int i = 0; i < 2; i++ )
     {
-        /* first collect linestrings related to HP, OB or DPM
+        /* first collect linestrings related to HP, OB, DPM and ZVB
            then collect rest of linestrings */
         if( i == 0 )
             osSQL.Printf("SELECT BP_ID,PORADOVE_CISLO_BODU,PARAMETRY_SPOJENI,_rowid_ FROM '%s' WHERE "
-                         "HP_ID IS NOT NULL OR OB_ID IS NOT NULL OR DPM_ID IS NOT NULL "
-                         "ORDER BY HP_ID,OB_ID,DPM_ID,PORADOVE_CISLO_BODU", m_pszName);
+                         "HP_ID IS NOT NULL OR OB_ID IS NOT NULL OR DPM_ID IS NOT NULL OR ZVB_ID IS NOT NULL "
+                         "ORDER BY HP_ID,OB_ID,DPM_ID,ZVB_ID,PORADOVE_CISLO_BODU", m_pszName);
         else
             osSQL.Printf("SELECT BP_ID,PORADOVE_CISLO_BODU,PARAMETRY_SPOJENI,_rowid_ FROM '%s' WHERE "
-                         "OB_ID IS NULL AND HP_ID IS NULL AND DPM_ID IS NULL "
+                         "OB_ID IS NULL AND HP_ID IS NULL AND DPM_ID IS NULL AND ZVB_ID IS NULL "
                          "ORDER BY ID,PORADOVE_CISLO_BODU", m_pszName);
 
         sqlite3_stmt *hStmt = poReader->PrepareStatement(osSQL.c_str());
@@ -248,40 +250,43 @@ int VFKDataBlockSQLite::LoadGeometryLineStringSBP()
             poReader->ExecuteSQL("BEGIN");
 
         std::vector<int> rowIdFeat;
-        CPLString szFType;
+        CPLString osFType;
         OGRLineString oOGRLine;
 
         while(poReader->ExecuteSQL(hStmt) == OGRERR_NONE) {
             // read values
             const GUIntBig id = sqlite3_column_int64(hStmt, 0);
             const GUIntBig ipcb  = sqlite3_column_int64(hStmt, 1);
-            szFType = (char *) sqlite3_column_text(hStmt, 2);
+            const char* pszFType = reinterpret_cast<const char*>(
+                sqlite3_column_text(hStmt, 2));
+            osFType = pszFType ? pszFType : "";
             int rowId = sqlite3_column_int(hStmt, 3);
 
             if (ipcb == 1) {
                 VFKFeatureSQLite *poFeature =
                     (VFKFeatureSQLite *) GetFeatureByIndex(iIdx);
-                if( poFeature == NULL )
+                if( poFeature == nullptr )
                 {
                     CPLError(CE_Failure, CPLE_AppDefined,
                              "Cannot retrieve feature %d", iIdx);
+                    sqlite3_finalize(hStmt);
                     break;
                 }
                 poFeature->SetRowId(rowId);
 
                 /* set geometry & reset */
-                CPLString szFTypeLine;
+                CPLString osFTypeLine;
                 if( poLine &&
                     !SetGeometryLineString(
                         poLine, &oOGRLine,
-                        bValid, szFTypeLine, rowIdFeat, nGeometries) )
+                        bValid, osFTypeLine, rowIdFeat, nGeometries) )
                 {
                     nInvalid++;
                 }
 
                 bValid = true;
                 poLine = poFeature;
-                szFTypeLine = szFType;
+                osFTypeLine = osFType;
                 iIdx++;
             }
 
@@ -289,9 +294,9 @@ int VFKDataBlockSQLite::LoadGeometryLineStringSBP()
                 (VFKFeatureSQLite *) poDataBlockPoints->GetFeature("ID", id);
             if( poPoint )
             {
-                OGRPoint *pt = (OGRPoint *) poPoint->GetGeometry();
+                OGRGeometry *pt = poPoint->GetGeometry();
                 if (pt) {
-                    oOGRLine.addPoint(pt);
+                    oOGRLine.addPoint(pt->toPoint());
                 }
                 else
                 {
@@ -316,11 +321,11 @@ int VFKDataBlockSQLite::LoadGeometryLineStringSBP()
         if( poLine &&
             !SetGeometryLineString(
                 poLine, &oOGRLine,
-                bValid, szFType.c_str(), rowIdFeat, nGeometries) )
+                bValid, osFType.c_str(), rowIdFeat, nGeometries) )
         {
             nInvalid++;
         }
-        poLine = NULL;
+        poLine = nullptr;
 
         if (poReader->IsSpatial())
             poReader->ExecuteSQL("COMMIT");
@@ -333,7 +338,7 @@ int VFKDataBlockSQLite::LoadGeometryLineStringSBP()
 }
 
 /*!
-  \brief Load geometry (linestring HP/DPM layer)
+  \brief Load geometry (linestring HP/DPM/ZVB layer)
 
   \return number of invalid features
 */
@@ -344,7 +349,7 @@ int VFKDataBlockSQLite::LoadGeometryLineStringHP()
 
     VFKDataBlockSQLite *poDataBlockLines =
         (VFKDataBlockSQLite *) m_poReader->GetDataBlock("SBP");
-    if (NULL == poDataBlockLines) {
+    if (nullptr == poDataBlockLines) {
         CPLError(CE_Failure, CPLE_FileIO,
                  "Data block %s not found.", m_pszName);
         return nInvalid;
@@ -380,27 +385,30 @@ int VFKDataBlockSQLite::LoadGeometryLineStringHP()
     {
         /* read values */
         vrValue[0] = sqlite3_column_int64(hStmt, 0);
-        const long iFID = static_cast<long>(sqlite3_column_int64(hStmt, 1));
+        const GIntBig iFID = sqlite3_column_int64(hStmt, 1);
         const int rowId = sqlite3_column_int(hStmt, 2);
 
         VFKFeatureSQLite *poFeature =
             (VFKFeatureSQLite *) GetFeatureByIndex(rowId - 1);
-        CPLAssert(NULL != poFeature && poFeature->GetFID() == iFID);
+        if( poFeature == nullptr || poFeature->GetFID() != iFID )
+        {
+            continue;
+        }
 
         VFKFeatureSQLite *poLine =
             poDataBlockLines->GetFeature(vrColumn, vrValue, 2, TRUE);
 
-        OGRGeometry *poOgrGeometry = NULL;
+        OGRGeometry *poOgrGeometry = nullptr;
         if( !poLine )
         {
-            poOgrGeometry = NULL;
+            poOgrGeometry = nullptr;
         }
         else
         {
             poOgrGeometry = poLine->GetGeometry();
         }
         if (!poOgrGeometry || !poFeature->SetGeometry(poOgrGeometry)) {
-            CPLDebug("OGR-VFK", "VFKDataBlockSQLite::LoadGeometryLineStringHP(): name=%s fid=%ld "
+            CPLDebug("OGR-VFK", "VFKDataBlockSQLite::LoadGeometryLineStringHP(): name=%s fid=" CPL_FRMT_GIB " "
                      "id=" CPL_FRMT_GUIB " -> %s geometry", m_pszName, iFID, vrValue[0],
                      poOgrGeometry ? "invalid" : "empty");
             nInvalid++;
@@ -432,8 +440,8 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
 {
     VFKReaderSQLite *poReader = (VFKReaderSQLite*) m_poReader;
 
-    VFKDataBlockSQLite *poDataBlockLines1 = NULL;
-    VFKDataBlockSQLite *poDataBlockLines2 = NULL;
+    VFKDataBlockSQLite *poDataBlockLines1 = nullptr;
+    VFKDataBlockSQLite *poDataBlockLines2 = nullptr;
     bool bIsPar = false;
     if (EQUAL (m_pszName, "PAR")) {
         poDataBlockLines1 = (VFKDataBlockSQLite *) m_poReader->GetDataBlock("HP");
@@ -445,14 +453,14 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
         poDataBlockLines2 = (VFKDataBlockSQLite *) m_poReader->GetDataBlock("SBP");
         bIsPar = false;
     }
-    if( NULL == poDataBlockLines1 )
+    if( nullptr == poDataBlockLines1 )
     {
         CPLError(CE_Warning, CPLE_FileIO,
                  "Data block %s not found. Unable to build geometry for %s.",
                  bIsPar ? "HP" : "OB", m_pszName);
         return -1;
     }
-    if( NULL == poDataBlockLines2 )
+    if( nullptr == poDataBlockLines2 )
     {
         CPLError(CE_Warning, CPLE_FileIO,
                  "Data block %s not found. Unable to build geometry for %s.",
@@ -466,7 +474,7 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
     if( LoadGeometryFromDB() )  // Try to load geometry from DB.
         return 0;
 
-    const char *vrColumn[2] = { NULL, NULL };
+    const char *vrColumn[2] = { nullptr, nullptr };
     GUIntBig vrValue[2] = { 0, 0 };
     if (bIsPar) {
         vrColumn[0] = "PAR_ID_1";
@@ -502,7 +510,10 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
 
         VFKFeatureSQLite *poFeature =
             (VFKFeatureSQLite *) GetFeatureByIndex(rowId - 1);
-        CPLAssert(NULL != poFeature && poFeature->GetFID() == iFID);
+        if( poFeature == nullptr || poFeature->GetFID() != iFID )
+        {
+            continue;
+        }
 
         if( bIsPar )
         {
@@ -543,6 +554,13 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
 
         /* clear */
         ogrPolygon.empty();
+
+        /* free ring list */
+        for (PointListArray::iterator iRing = poRingList.begin(), eRing = poRingList.end();
+            iRing != eRing; ++iRing) {
+            delete (*iRing);
+            *iRing = nullptr;
+        }
         poRingList.clear();
 
         /* collect rings from lines */
@@ -556,8 +574,8 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
             int i = 1;
             for (VFKFeatureSQLiteList::iterator iHp = poLineList.begin(), eHp = poLineList.end();
                  iHp != eHp; ++iHp, ++i) {
-                const OGRLineString *pLine = (OGRLineString *) (*iHp)->GetGeometry();
-                if (pLine && AppendLineToRing(&poRingList, pLine, bNewRing)) {
+                auto pGeom = (*iHp)->GetGeometry();
+                if (pGeom && AppendLineToRing(&poRingList, pGeom->toLineString(), bNewRing)) {
                     bFound = true;
                     poLineList.erase(iHp);
                     break;
@@ -578,7 +596,7 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
 
         /* build rings */
         poLinearRingList.clear();
-        OGRLinearRing *poOgrRing = NULL;
+        OGRLinearRing *poOgrRing = nullptr;
         int i = 1;
         for( PointListArray::const_iterator iRing = poRingList.begin(),
                  eRing = poRingList.end();
@@ -589,7 +607,7 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
 
             poLinearRingList.push_back(new OGRLinearRing());
             poOgrRing = poLinearRingList.back();
-            CPLAssert(NULL != poOgrRing);
+            CPLAssert(nullptr != poOgrRing);
 
             for( PointList::iterator iPoint = poList->begin(),
                      ePoint = poList->end();
@@ -657,7 +675,7 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
                 }
             }
             delete poOgrRing;
-            *iRing = NULL;
+            *iRing = nullptr;
         }
 
         /* set polygon */
@@ -678,7 +696,7 @@ int VFKDataBlockSQLite::LoadGeometryPolygon()
     for (PointListArray::iterator iRing = poRingList.begin(), eRing = poRingList.end();
          iRing != eRing; ++iRing) {
         delete (*iRing);
-        *iRing = NULL;
+        *iRing = nullptr;
     }
 
     CPLDebug("OGR-VFK", "%s: nolines = %d norings = %d",
@@ -709,7 +727,7 @@ IVFKFeature *VFKDataBlockSQLite::GetFeature(GIntBig nFID)
     }
 
     if (nFID < 1 || nFID > m_nFeatureCount)
-        return NULL;
+        return nullptr;
 
     if( m_bGeometryPerBlock && !m_bGeometry )
     {
@@ -736,7 +754,7 @@ IVFKFeature *VFKDataBlockSQLite::GetFeature(GIntBig nFID)
 }
 
 /*!
-  \brief Get first found feature based on it's property
+  \brief Get first found feature based on its property
 
   \param column property name
   \param value property value
@@ -762,19 +780,19 @@ VFKFeatureSQLite *VFKDataBlockSQLite::GetFeature(const char *column, GUIntBig va
 
     sqlite3_stmt *hStmt = poReader->PrepareStatement(osSQL.c_str());
     if (poReader->ExecuteSQL(hStmt) != OGRERR_NONE)
-        return NULL;
+        return nullptr;
 
     const int idx = sqlite3_column_int(hStmt, 0) - 1;
     sqlite3_finalize(hStmt);
 
     if (idx < 0 || idx >= m_nFeatureCount) // ? assert
-        return NULL;
+        return nullptr;
 
     return (VFKFeatureSQLite *) GetFeatureByIndex(idx);
 }
 
 /*!
-  \brief Get first found feature based on it's properties (AND)
+  \brief Get first found feature based on its properties (AND)
 
   \param column array of property names
   \param value array of property values
@@ -808,13 +826,13 @@ VFKFeatureSQLite *VFKDataBlockSQLite::GetFeature(const char **column, GUIntBig *
 
     sqlite3_stmt *hStmt = poReader->PrepareStatement(osSQL.c_str());
     if (poReader->ExecuteSQL(hStmt) != OGRERR_NONE)
-        return NULL;
+        return nullptr;
 
     int idx = sqlite3_column_int(hStmt, 0) - 1; /* rowid starts at 1 */
     sqlite3_finalize(hStmt);
 
     if (idx < 0 || idx >= m_nFeatureCount) // ? assert
-        return NULL;
+        return nullptr;
 
     return (VFKFeatureSQLite *) GetFeatureByIndex(idx);
 }
@@ -850,7 +868,16 @@ VFKFeatureSQLiteList VFKDataBlockSQLite::GetFeatures(const char **column, GUIntB
     sqlite3_stmt *hStmt = poReader->PrepareStatement(osSQL.c_str());
     while (poReader->ExecuteSQL(hStmt) == OGRERR_NONE) {
         const int iRowId = sqlite3_column_int(hStmt, 0);
-        fList.push_back((VFKFeatureSQLite *)GetFeatureByIndex(iRowId - 1));
+        VFKFeatureSQLite* poFeature = dynamic_cast<VFKFeatureSQLite*>(
+            GetFeatureByIndex(iRowId - 1));
+        if( poFeature == nullptr )
+        {
+            CPLError(CE_Failure, CPLE_AppDefined,
+                     "Cannot retrieve feature %d", iRowId);
+            sqlite3_finalize(hStmt);
+            return VFKFeatureSQLiteList();
+        }
+        fList.push_back(poFeature);
     }
 
     return fList;
@@ -869,10 +896,15 @@ OGRErr VFKDataBlockSQLite::SaveGeometryToDB(const OGRGeometry *poGeom, int iRowI
     int        rc, nWKBLen;
     CPLString  osSQL;
 
-    sqlite3_stmt *hStmt = NULL;
+    sqlite3_stmt *hStmt = nullptr;
 
     VFKReaderSQLite *poReader = (VFKReaderSQLite*) m_poReader;
 
+    /* check if geometry column exists (see SUPPRESS_GEOMETRY open
+       option) */
+    if ( AddGeometryColumn() != OGRERR_NONE )
+        return OGRERR_FAILURE;
+
     if (poGeom) {
         nWKBLen = poGeom->WkbSize();
         GByte *pabyWKB = (GByte *) CPLMalloc(nWKBLen + 1);
@@ -943,19 +975,20 @@ bool VFKDataBlockSQLite::LoadGeometryFromDB()
 
     while(poReader->ExecuteSQL(hStmt) == OGRERR_NONE) {
         rowId++; // =sqlite3_column_int(hStmt, 1);
-#ifdef DEBUG
         const GIntBig iFID = sqlite3_column_int64(hStmt, 2);
-#endif
-
-        VFKFeatureSQLite *poFeature = (VFKFeatureSQLite *) GetFeatureByIndex(rowId - 1);
-        CPLAssert(NULL != poFeature && poFeature->GetFID() == iFID);
+        VFKFeatureSQLite *poFeature = dynamic_cast<VFKFeatureSQLite *>(
+            GetFeatureByIndex(rowId - 1));
+        if( poFeature == nullptr || poFeature->GetFID() != iFID )
+        {
+            continue;
+        }
 
         // read geometry from DB
         const int nBytes = sqlite3_column_bytes(hStmt, 0);
-        OGRGeometry *poGeometry = NULL;
+        OGRGeometry *poGeometry = nullptr;
         if (nBytes > 0 &&
-            OGRGeometryFactory::createFromWkb((GByte*) sqlite3_column_blob(hStmt, 0),
-                                              NULL, &poGeometry, nBytes) == OGRERR_NONE) {
+            OGRGeometryFactory::createFromWkb(sqlite3_column_blob(hStmt, 0),
+                                              nullptr, &poGeometry, nBytes) == OGRERR_NONE) {
             nGeometriesCount++;
             if (!poFeature->SetGeometry(poGeometry)) {
                 nInvalid++;
@@ -1076,7 +1109,7 @@ const char *VFKDataBlockSQLite::GetKey() const
             return pszKey;
     }
 
-    return NULL;
+    return nullptr;
 }
 
 /*!
@@ -1095,3 +1128,26 @@ int VFKDataBlockSQLite::GetGeometrySQLType() const
 
     return 0; /* unknown geometry type */
 }
+
+/*!
+  \brief Add geometry column into table if not exists
+
+  \return OGRERR_NONE on success otherwise OGRERR_FAILURE
+ */
+OGRErr VFKDataBlockSQLite::AddGeometryColumn() const
+{
+    CPLString osSQL;
+
+    VFKReaderSQLite *poReader = (VFKReaderSQLite*) m_poReader;
+
+    osSQL.Printf("SELECT %s FROM %s LIMIT 0",
+                 GEOM_COLUMN, m_pszName);
+    if ( poReader->ExecuteSQL(osSQL.c_str(), CE_None) == OGRERR_FAILURE ) {
+        /* query failed, we assume that geometry column not exists */
+        osSQL.Printf("ALTER TABLE %s ADD COLUMN %s blob",
+                     m_pszName, GEOM_COLUMN);
+        return poReader->ExecuteSQL(osSQL.c_str());
+    }
+
+    return OGRERR_NONE;
+}
